#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_celpprunner
----------------------------------

Tests for `celpprunner` module.
"""

import unittest
import tempfile
import os
import os.path
import shutil
import gzip
from datetime import date

from d3r import celpprunner
from d3r.celpp.task import D3RParameters
from d3r.celpp import util
from d3r.celpp.task import D3RTask
from d3r.celpp.evaluation import EvaluationTaskFactory
from d3r.celpp.blastnfilter import BlastNFilterTask
from d3r.celpp.dataimport import DataImportTask
from d3r.celpp.makeblastdb import MakeBlastDBTask
from d3r.celpp.proteinligprep import ProteinLigPrepTask
from d3r.celpp.glide import GlideTask
from d3r.celpp.vina import AutoDockVinaTask
from d3r.celpp.challengedata import ChallengeDataTask
from d3r.celpp.chimeraprep import ChimeraProteinLigPrepTask


class DummyTask(D3RTask):
    """Dummy Task used for tests below
    """
    def __init__(self, path, args, error_message, can_run_return,
                 can_run_exception, run_exception):
        """Sets up a dummy task
        """
        super(DummyTask, self).__init__(path, args)
        self.set_error(error_message)
        self._can_run_return = can_run_return
        self._can_run_exception = can_run_exception
        self._run_exception = run_exception
        self._run_count = 0

    def can_run(self):
        if self._can_run_exception is not None:
            raise self._can_run_exception

        return self._can_run_return

    def run(self):
        self._run_count += 1
        if self._run_exception is not None:
            raise self._run_exception


class TestCelppRunner(unittest.TestCase):
    """Tests celpprunner command line script
    """
    param = D3RParameters()

    blast = BlastNFilterTask('/foo', param)
    BLAST_DIR_NAME = blast.get_dir_name()
    BLAST_NAME = blast.get_name()

    data = DataImportTask('/foo', param)
    IMPORT_DIR_NAME = data.get_dir_name()
    IMPORT_NAME = data.get_name()

    makedb = MakeBlastDBTask('/foo', param)
    MAKEDB_DIR_NAME = makedb.get_dir_name()
    MAKEDB_NAME = makedb.get_name()

    glide = GlideTask('/foo', param)
    GLIDE_DIR_NAME = glide.get_dir_name()

    prot = ProteinLigPrepTask('/foo', param)
    PROT_DIR_NAME = prot.get_dir_name()

    vina = AutoDockVinaTask('/foo', param)
    VINA_DIR_NAME = vina.get_dir_name()

    chall = ChallengeDataTask('/foo', param)
    CHALL_DIR_NAME = chall.get_dir_name()
    CHALL_NAME = chall.get_name()

    chimeraprep = ChimeraProteinLigPrepTask('/foo', param)
    CHIMERAPREP_DIR_NAME = chimeraprep.get_dir_name()

    def setUp(self):
        pass

    def test_get_lock(self):
        temp_dir = tempfile.mkdtemp()
        try:
            theargs = D3RParameters()
            theargs.latest_weekly = temp_dir

            # get the lock file which should work
            lock = celpprunner._get_lock(theargs, 'blast')
            expectedLockFile = os.path.join(temp_dir,
                                            'celpprunner.blast.lockpid')
            self.assertTrue(os.path.isfile(expectedLockFile))

            # try getting lock again which should also work
            lock = celpprunner._get_lock(theargs, 'blast')

            lock.release()
            self.assertFalse(os.path.isfile(expectedLockFile))
        finally:
            shutil.rmtree(temp_dir)

    def test_get_lock_where_lockfile_exists_and_process_is_running(self):
        temp_dir = tempfile.mkdtemp()
        try:
            theargs = D3RParameters()
            theargs.latest_weekly = temp_dir

            lockfile = os.path.join(temp_dir,
                                    'celpprunner.blast.lockpid')
            f = open(lockfile, 'w')
            pid = str(os.getppid())
            f.write(pid)
            f.flush()
            f.close()
            # get the lock file which should work
            try:
                celpprunner._get_lock(theargs, 'blast')
                self.fail('Expected Exception')
            except Exception as e:
                self.assertEqual(str(e), 'celpprunner with pid ' +
                                 pid + ' is running')
        finally:
            shutil.rmtree(temp_dir)

    def test_parse_arguments(self):
        theargs = ['--stage', 'blast', 'foo']
        result = celpprunner._parse_arguments('hi', theargs)
        self.assertEqual(result.stage, 'blast')
        self.assertEqual(result.celppdir, 'foo')
        self.assertEqual(result.email, None)
        self.assertEqual(result.summaryemail, None)
        self.assertEqual(result.loglevel, celpprunner.DEFAULT_LOG_LEVEL)
        self.assertEqual(result.blastnfilter, 'blastnfilter.py')
        self.assertEqual(result.proteinligprep, 'proteinligprep.py')
        self.assertEqual(result.evaluation, 'evaluate.py')
        self.assertEqual(result.makeblastdb, 'makeblastdb')
        self.assertEqual(result.genchallenge, 'genchallengedata.py')
        self.assertEqual(result.chimeraprep, 'chimera_proteinligprep.py')
        self.assertEqual(result.skipimportwait, False)
        self.assertEqual(result.importretry, 60)
        self.assertEqual(result.importsleep, 600)
        self.assertEqual(result.rdkitpython, '')
        self.assertEqual(result.summaryemail, None)
        self.assertEqual(result.postevaluation, 'post_evaluation.py')
        theargs = ['foo', '--stage', 'dock,glide', '--email', 'b@b.com,h@h',
                   '--log', 'ERROR',
                   '--blastnfilter', '/bin/blastnfilter.py',
                   '--proteinligprep', '/bin/proteinligprep.py',
                   '--postanalysis', '/bin/postanalysis.py',
                   '--glide', '/bin/glide.py',
                   '--vina', '/bin/vina.py',
                   '--customweekdir',
                   '--evaluation', '/bin/evaluation.py',
                   '--makeblastdb', '/bin/makeblastdb',
                   '--genchallenge', '/bin/gen.py',
                   '--chimeraprep', '/bin/chimeraprep.py',
                   '--skipimportwait',
                   '--importretry', '10',
                   '--importsleep', '30',
                   '--rdkitpython', '/usr/bin',
                   '--summaryemail', 'j@j,g@g',
                   '--postevaluation', '/bin/yo.py']
        result = celpprunner._parse_arguments('hi', theargs)
        self.assertEqual(result.stage, 'dock,glide')
        self.assertEqual(result.celppdir, 'foo')
        self.assertEqual(result.email, 'b@b.com,h@h')
        self.assertEqual(result.summaryemail, 'j@j,g@g')
        self.assertEqual(result.loglevel, 'ERROR')
        self.assertEqual(result.blastnfilter, '/bin/blastnfilter.py')
        self.assertEqual(result.proteinligprep, '/bin/proteinligprep.py')
        self.assertEquals(result.postanalysis, '/bin/postanalysis.py')
        self.assertEquals(result.glide, '/bin/glide.py')
        self.assertEquals(result.evaluation, '/bin/evaluation.py')
        self.assertEquals(result.customweekdir, True)
        self.assertEqual(result.makeblastdb, '/bin/makeblastdb')
        self.assertEqual(result.vina, '/bin/vina.py')
        self.assertEqual(result.genchallenge, '/bin/gen.py')
        self.assertEqual(result.chimeraprep, '/bin/chimeraprep.py')
        self.assertEqual(result.skipimportwait, True)
        self.assertEqual(result.importretry, 10)
        self.assertEqual(result.importsleep, 30)
        self.assertEqual(result.rdkitpython, '/usr/bin')
        self.assertEqual(result.postevaluation, '/bin/yo.py')

    def test_run_tasks_passing_none_and_empty_list(self):
        self.assertEquals(celpprunner.run_tasks(None), 3)
        task_list = []
        self.assertEquals(celpprunner.run_tasks(task_list), 2)

    def test_run_one_successful_task(self):
        success_task = DummyTask(D3RParameters(), 'foo', None, True, None,
                                 None)
        success_task.set_name('dummy')
        task_list = []
        task_list.append(success_task)
        self.assertEquals(celpprunner.run_tasks(task_list), 0)

    def test_run_one_fail_task_with_error_message(self):
        task = DummyTask(D3RParameters(), 'foo', 'someerror', True, None, None)
        task.set_name('dummy')
        task_list = []
        task_list.append(task)
        self.assertEquals(celpprunner.run_tasks(task_list), 1)
        self.assertEquals(task.get_error(), 'someerror')

    def test_run_one_fail_task_with_exception_and_no_message(self):
        task = DummyTask(D3RParameters(), 'foo', None, True,
                         None, Exception('hi'))
        task.set_name('dummy')
        task_list = []
        task_list.append(task)
        self.assertEquals(celpprunner.run_tasks(task_list), 1)
        self.assertEquals(task.get_error(),
                          'Caught Exception running task: hi')

    def test_run_two_tasks_success(self):
        task_list = []
        task = DummyTask(D3RParameters(), 'foo', None, True, None, None)
        task.set_name('dummy')
        task_list.append(task)
        task_list.append(task)

        self.assertEquals(celpprunner.run_tasks(task_list), 0)
        self.assertEquals(task._run_count, 2)

    def test_run_two_tasks_second_task_has_error(self):
        task_list = []
        task = DummyTask(D3RParameters(), 'foo', None, True, None, None)
        task.set_name('dummy')
        task_list.append(task)

        task_two = DummyTask(D3RParameters(), 'foo', None, True,
                             None, Exception('hi'))
        task_two.set_name('dummy')
        task_list.append(task_two)

        self.assertEquals(celpprunner.run_tasks(task_list), 1)
        self.assertEquals(task._run_count, 1)
        self.assertEquals(task_two._run_count, 1)
        self.assertEquals(task_two.get_error(),
                          'Caught Exception running task: hi')

    def test_run_two_tasks_first_task_has_error(self):
        task_list = []
        task = DummyTask(D3RParameters(), 'foo', None, True, None,
                         Exception('hi'))
        task.set_name('dummy')
        task_list.append(task)

        task_two = DummyTask(D3RParameters(), 'foo', None, True, None,
                             None)
        task_two.set_name('dummy')
        task_list.append(task_two)

        self.assertEquals(celpprunner.run_tasks(task_list), 1)
        self.assertEquals(task.get_error(),
                          'Caught Exception running task: hi')

        self.assertEquals(task._run_count, 1)
        self.assertEquals(task_two._run_count, 1)

    def test_get_set_of_email_address_from_email_flags(self):

        params = D3RParameters()
        res = celpprunner._get_set_of_email_address_from_email_flags(params)
        self.assertEqual(res, None)

        # email set
        params.email = 'b@b.com'
        res = celpprunner._get_set_of_email_address_from_email_flags(params)
        self.assertEqual(res, ['b@b.com'])

        # email set w dup
        params.email = 'b@b.com,j@j.com,b@b.com'
        res = celpprunner._get_set_of_email_address_from_email_flags(params)
        self.assertEqual(res, ['b@b.com', 'j@j.com'])

        # summary set only
        params.email = None
        params.summaryemail = 'b@b.com'
        res = celpprunner._get_set_of_email_address_from_email_flags(params)
        self.assertEqual(res, ['b@b.com'])

        # summary w dups
        params.summaryemail = 'b@b.com,j@j.com,b@b.com'
        res = celpprunner._get_set_of_email_address_from_email_flags(params)
        self.assertEqual(res, ['b@b.com', 'j@j.com'])

        # both set
        params.email = 'h@h.com,j@j.com'
        res = celpprunner._get_set_of_email_address_from_email_flags(params)
        self.assertEqual(res, ['b@b.com', 'h@h.com', 'j@j.com'])

    def test_get_task_list_for_stage_with_invalid_stage_name(self):

        try:
            celpprunner.get_task_list_for_stage(D3RParameters(), None)
            self.fail('Expected exception')
        except NotImplementedError as e:
            self.assertEquals(e.message, 'stage_name is None')

        try:
            celpprunner.get_task_list_for_stage(D3RParameters(), '')
            self.fail('Expected exception')
        except NotImplementedError as e:
            self.assertEquals(e.message, 'uh oh no tasks for  stage')

        try:
            celpprunner.get_task_list_for_stage(D3RParameters(), 'foo')
            self.fail('Expected exception')
        except NotImplementedError as e:
            self.assertEquals(e.message, 'uh oh no tasks for foo stage')

    def test_get_task_list_for_stage_with_valid_stages(self):
        params = D3RParameters()
        params.latest_weekly = 'foo'
        task_list = celpprunner.get_task_list_for_stage(params, 'blast')
        self.assertEquals(len(task_list), 1)

        self.assertEquals(task_list[0].get_dir(),
                          os.path.join('foo', TestCelppRunner.BLAST_DIR_NAME))

        task_list = celpprunner.get_task_list_for_stage(params,
                                                        'proteinligprep')
        self.assertEquals(len(task_list), 1)

        self.assertEquals(task_list[0].get_dir(),
                          os.path.join('foo', TestCelppRunner.PROT_DIR_NAME))

        task_list = celpprunner.get_task_list_for_stage(params, 'import')
        self.assertEquals(len(task_list), 1)
        self.assertEquals(task_list[0].get_dir(),
                          os.path.join('foo', TestCelppRunner.IMPORT_DIR_NAME))

        task_list = celpprunner.get_task_list_for_stage(params, 'glide')
        self.assertEquals(len(task_list), 1)
        self.assertEquals(task_list[0].get_dir(),
                          os.path.join('foo', TestCelppRunner.GLIDE_DIR_NAME))

        task_list = celpprunner.get_task_list_for_stage(params, 'vina')
        self.assertEquals(len(task_list), 1)
        self.assertEquals(task_list[0].get_dir(),
                          os.path.join('foo', TestCelppRunner.VINA_DIR_NAME))

        task_list = celpprunner.get_task_list_for_stage(params,
                                                        'challengedata')
        self.assertEquals(len(task_list), 1)
        self.assertEquals(task_list[0].get_dir(),
                          os.path.join('foo', TestCelppRunner.CHALL_DIR_NAME))

        task_list = celpprunner.get_task_list_for_stage(params,
                                                        'chimeraprep')
        self.assertEquals(len(task_list), 1)
        self.assertEquals(task_list[0].get_dir(),
                          os.path.join('foo',
                                       TestCelppRunner.CHIMERAPREP_DIR_NAME))

    def test_get_task_list_for_stage_createchallenge(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.latest_weekly = temp_dir
            task_list = celpprunner.get_task_list_for_stage(
                params, celpprunner.CREATE_CHALLENGE)
            self.assertEqual(len(task_list), 4)
            self.assertEqual(task_list[0].get_name(),
                             TestCelppRunner.MAKEDB_NAME)
            self.assertEqual(task_list[1].get_name(),
                             TestCelppRunner.IMPORT_NAME)
            self.assertEqual(task_list[2].get_name(),
                             TestCelppRunner.BLAST_NAME)
            self.assertEqual(task_list[3].get_name(),
                             TestCelppRunner.CHALL_NAME)

        finally:
            shutil.rmtree(temp_dir)

    def test_get_task_list_for_stage_for_scoring_stage_with_nonefound(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.latest_weekly = temp_dir
            try:
                celpprunner.get_task_list_for_stage(params, 'evaluation')
            except NotImplementedError as e:
                self.assertEqual(e.message,
                                 'uh oh no tasks for evaluation stage')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_task_list_for_stage_for_scoring_stage_with_onefound(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.latest_weekly = temp_dir
            glidedir = os.path.join(temp_dir,
                                    EvaluationTaskFactory.DOCKSTAGE_PREFIX +
                                    'glide')
            os.mkdir(glidedir)
            open(os.path.join(glidedir, D3RTask.COMPLETE_FILE), 'a').close()
            task_list = celpprunner.get_task_list_for_stage(params,
                                                            'evaluation')
            self.assertEqual(len(task_list), 1)
            self.assertEqual(task_list[0].get_name(), 'glide.evaluation')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_task_list_for_stage_for_scoring_stage_with_twofound(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.latest_weekly = temp_dir
            glidedir = os.path.join(temp_dir,
                                    EvaluationTaskFactory.DOCKSTAGE_PREFIX +
                                    'glide')
            os.mkdir(glidedir)
            open(os.path.join(glidedir, D3RTask.COMPLETE_FILE), 'a').close()
            freddir = os.path.join(temp_dir,
                                   EvaluationTaskFactory.DOCKSTAGE_PREFIX +
                                   'fred')
            os.mkdir(freddir)
            open(os.path.join(freddir, D3RTask.COMPLETE_FILE), 'a').close()

            task_list = celpprunner.get_task_list_for_stage(params,
                                                            'evaluation')
            self.assertEqual(len(task_list), 2)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_stages_no_weekly_datasetfound(self):
        temp_dir = tempfile.mkdtemp()

        try:
            theargs = D3RParameters()
            theargs.celppdir = temp_dir
            self.assertEqual(celpprunner.run_stages(theargs), 0)

        finally:
            shutil.rmtree(temp_dir)

    def test_run_stages_invalid_stage(self):
        temp_dir = tempfile.mkdtemp()
        try:
            theargs = D3RParameters()
            theargs.celppdir = temp_dir
            theargs.stage = 'foo'
            os.mkdir(os.path.join(temp_dir, '2015'))
            os.mkdir(os.path.join(temp_dir, '2015', 'dataset.week.1'))
            try:
                celpprunner.run_stages(theargs)
            except NotImplementedError as e:
                self.assertEquals(e.message, 'uh oh no tasks for foo stage')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_stages_blast_stage_data_import_missing(self):
        temp_dir = tempfile.mkdtemp()
        try:
            theargs = D3RParameters()
            theargs.celppdir = temp_dir
            theargs.stage = 'blast'
            os.mkdir(os.path.join(temp_dir, '2015'))
            os.mkdir(os.path.join(temp_dir, '2015', 'dataset.week.1'))
            makedb_dir = os.path.join(temp_dir, '2015', 'dataset.week.1',
                                      TestCelppRunner.MAKEDB_DIR_NAME)
            os.makedirs(makedb_dir)
            open(os.path.join(makedb_dir, 'complete'), 'a').close()
            self.assertEquals(celpprunner.run_stages(theargs), 1)

        finally:
            shutil.rmtree(temp_dir)

    def test_run_stages_blast(self):
        temp_dir = tempfile.mkdtemp()

        try:
            theargs = D3RParameters()
            theargs.celppdir = os.path.join(temp_dir)
            theargs.stage = 'blast'
            theargs.pdbdb = '/pdbdb'

            makedb_dir = os.path.join(temp_dir, '2015', 'dataset.week.1',
                                      TestCelppRunner.MAKEDB_DIR_NAME)
            os.makedirs(makedb_dir)
            open(os.path.join(makedb_dir, 'complete'), 'a').close()

            d_import_dir = os.path.join(temp_dir, '2015', 'dataset.week.1',
                                        TestCelppRunner.IMPORT_DIR_NAME)
            os.makedirs(d_import_dir)
            open(os.path.join(d_import_dir, 'complete'), 'a').close()

            theargs.blastnfilter = 'echo'
            theargs.postanalysis = 'true'
            self.assertEqual(celpprunner.run_stages(theargs), 0)

        finally:
            shutil.rmtree(temp_dir)

    def test_run_stages_blast_has_error(self):
        temp_dir = tempfile.mkdtemp()

        try:
            theargs = D3RParameters()
            theargs.celppdir = os.path.join(temp_dir)
            theargs.stage = 'blast'
            os.mkdir(os.path.join(temp_dir, TestCelppRunner.MAKEDB_DIR_NAME))
            open(os.path.join(temp_dir, TestCelppRunner.MAKEDB_DIR_NAME,
                              'error'),
                 'a').close()
            os.mkdir(os.path.join(temp_dir, '2015'))
            os.mkdir(os.path.join(temp_dir, '2015', 'dataset.week.1'))
            self.assertEqual(celpprunner.run_stages(theargs), 1)

        finally:
            shutil.rmtree(temp_dir)

    def test_run_stages_challenge_and_proteinligprep_no_error(self):
        temp_dir = tempfile.mkdtemp()

        try:
            theargs = D3RParameters()
            theargs.pdbdb = '/pdbdb'
            theargs.celppdir = os.path.join(temp_dir)
            theargs.stage = 'challengedata,proteinligprep'

            blastdb_dir = os.path.join(temp_dir, '2015', 'dataset.week.1',
                                       TestCelppRunner.BLAST_DIR_NAME)
            os.makedirs(blastdb_dir)
            open(os.path.join(blastdb_dir, 'complete'), 'a').close()

            theargs.proteinligprep = 'echo'
            theargs.genchallenge = 'echo'

            self.assertEqual(celpprunner.run_stages(theargs), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_stages_blast_and_proteinligprep_blast_has_error(self):
        temp_dir = tempfile.mkdtemp()

        try:
            theargs = D3RParameters()
            theargs.celppdir = os.path.join(temp_dir)
            theargs.stage = 'blast,proteinligprep'
            os.mkdir(os.path.join(temp_dir, TestCelppRunner.MAKEDB_DIR_NAME))
            open(os.path.join(temp_dir, TestCelppRunner.MAKEDB_DIR_NAME,
                              'complete'),
                 'a').close()
            d_import_dir = os.path.join(temp_dir, '2015', 'dataset.week.1',
                                        TestCelppRunner.IMPORT_DIR_NAME)
            os.makedirs(d_import_dir)
            open(os.path.join(d_import_dir, D3RTask.ERROR_FILE), 'a').close()
            theargs.blastnfilter = 'echo'
            theargs.proteinligprep = 'echo'
            self.assertEqual(celpprunner.run_stages(theargs), 1)

        finally:
            shutil.rmtree(temp_dir)

    def test_run_stages_makedb_blast_chall_proteinligprep_glide_no_error(self):
        temp_dir = tempfile.mkdtemp()

        try:
            theargs = D3RParameters()
            theargs.pdbdb = '/pdbdb'
            theargs.celppdir = os.path.join(temp_dir)

            theargs.stage = 'makedb,blast,challengedata,proteinligprep,glide'

            d_import_dir = os.path.join(temp_dir, '2015', 'dataset.week.1',
                                        TestCelppRunner.IMPORT_DIR_NAME)
            os.makedirs(d_import_dir)
            open(os.path.join(d_import_dir,
                              D3RTask.COMPLETE_FILE), 'a').close()

            fakegz = os.path.join(temp_dir, 'fake.gz')

            f = gzip.open(fakegz, 'wb')
            f.write('hello\n')
            f.flush()
            f.close()

            theargs.pdbsequrl = 'file://'+fakegz

            theargs.makeblastdb = 'echo'
            theargs.blastnfilter = 'echo'
            theargs.postanalysis = 'true'
            theargs.proteinligprep = 'echo'
            theargs.glide = 'echo'
            theargs.genchallenge = 'echo'
            self.assertEqual(celpprunner.run_stages(theargs), 0)

        finally:
            shutil.rmtree(temp_dir)

    def test_run_stages_makedb_through_glide(self):
        """This should test the following stages will run
           makedb,import,blast,challengedata,proteinligprep,glide,vina
        """
        temp_dir = tempfile.mkdtemp()

        try:
            theargs = D3RParameters()
            theargs.pdbdb = '/pdbdb'
            theargs.celppdir = os.path.join(temp_dir)

            theargs.stage = 'makedb,import,blast,challengedata,proteinligprep,' \
                            'chimeraprep,glide,vina'

            d_import_dir = os.path.join(temp_dir, '2015', 'dataset.week.1',
                                        TestCelppRunner.IMPORT_DIR_NAME)
            os.makedirs(d_import_dir)
            open(os.path.join(d_import_dir,
                              D3RTask.COMPLETE_FILE), 'a').close()

            fakegz = os.path.join(temp_dir, 'fake.gz')

            f = gzip.open(fakegz, 'wb')
            f.write('hello\n')
            f.flush()
            f.close()

            theargs.pdbsequrl = 'file://' + fakegz
            theargs.pdbfileurl = 'file://' + fakegz

            theargs.compinchi = 'file://' + fakegz
            theargs.version = '1.0.0'
            theargs.makeblastdb = 'echo'
            theargs.blastnfilter = 'echo'
            theargs.postanalysis = 'true'
            theargs.proteinligprep = 'echo'
            theargs.glide = 'echo'
            theargs.vina = 'echo'
            theargs.genchallenge = 'echo'
            theargs.chimeraprep = 'echo'
            self.assertEqual(celpprunner.run_stages(theargs), 0)

        finally:
            shutil.rmtree(temp_dir)

    def test_get_task_list_for_stage_extsubmission(self):
        temp_dir = tempfile.mkdtemp()
        try:
            theargs = D3RParameters()
            theargs.pdbdb = '/pdbdb'
            theargs.latest_weekly = temp_dir
            theargs.stage = 'extsubmission'
            try:
                celpprunner.get_task_list_for_stage(theargs, 'extsubmission')
                self.fail('expected NotImplementedError')
            except NotImplementedError:
                pass

        finally:
            shutil.rmtree(temp_dir)

    def test_get_task_list_for_stage_postevaluation(self):
        temp_dir = tempfile.mkdtemp()
        try:
            theargs = D3RParameters()
            theargs.latest_weekly = temp_dir
            theargs.stage = celpprunner.POST_EVAL
            t_list = celpprunner.get_task_list_for_stage(theargs,
                                                         theargs.stage)
            self.assertEqual(len(t_list), 1)
            self.assertEqual(t_list[0].get_name(), 'postevaluation')

            # try this time with email set
            theargs.email = 'b@b.com,j@j.com'
            t_list = celpprunner.get_task_list_for_stage(theargs,
                                                         theargs.stage)
            self.assertEqual(len(t_list), 1)
            self.assertEqual(t_list[0].get_name(), 'postevaluation')

            # try this time with both set
            theargs.email = 'b@b.com,j@j.com'
            theargs.summaryemail = 'b@b.com,h@h.com'
            t_list = celpprunner.get_task_list_for_stage(theargs,
                                                         theargs.stage)
            self.assertEqual(len(t_list), 1)
            self.assertEqual(t_list[0].get_name(), 'postevaluation')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_stages_createweekdir_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            theargs = D3RParameters()
            theargs.celppdir = os.path.join(temp_dir)
            theargs.createweekdir = True
            theargs.stage = ''
            d = date.today()
            celp_week = util.get_celpp_week_of_year_from_date(d)
            try:
                self.assertEquals(celpprunner.run_stages(theargs), 0)
                self.fail('Expected NotImplementedError')
            except NotImplementedError:
                pass

            expected_dir = os.path.join(temp_dir, str(celp_week[1]),
                                        'dataset.week.' +
                                        str(celp_week[0]))
            self.assertEquals(os.path.isdir(expected_dir), True)

        finally:
            shutil.rmtree(temp_dir)

    def test_run_stages_customweekdir_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            theargs = D3RParameters()
            theargs.celppdir = temp_dir
            theargs.customweekdir = True
            theargs.createweekdir = True
            theargs.stage = ''
            try:
                self.assertEquals(celpprunner.run_stages(theargs), 0)
                self.fail('Expected NotImplementedError')
            except NotImplementedError:
                pass

        finally:
            shutil.rmtree(temp_dir)

    def test_main_success(self):
        temp_dir = tempfile.mkdtemp()

        try:
            theargs = ['celpprunner.py', '--stage',
                       'blast', '--pdbdb', '/pdbdb',
                       '--blastnfilter', 'echo',
                       '--postanalysis', 'true',
                       temp_dir]

            makedb_dir = os.path.join(temp_dir, '2015', 'dataset.week.1',
                                      TestCelppRunner.MAKEDB_DIR_NAME)
            os.makedirs(makedb_dir)
            open(os.path.join(makedb_dir, 'complete'), 'a').close()

            d_import_dir = os.path.join(temp_dir, '2015', 'dataset.week.1',
                                        TestCelppRunner.IMPORT_DIR_NAME)
            os.makedirs(d_import_dir)
            open(os.path.join(d_import_dir, 'complete'), 'a').close()

            self.assertEqual(celpprunner.main(theargs), 0)

        finally:
            shutil.rmtree(temp_dir)

    def test_main_where_run_stages_raises_error(self):
        temp_dir = tempfile.mkdtemp()

        try:
            theargs = ['celpprunner.py', '--stage',
                       'foo', os.path.join(temp_dir, 'notexistdir')]
            self.assertEqual(celpprunner.main(theargs), 2)

        finally:
            shutil.rmtree(temp_dir)

    def tearDown(self):
        pass


if __name__ == '__main__':
    unittest.main()



__author__ = 'robswift'

import logging

logger = logging.getLogger(__name__)


class HitSequence(object):
    """

    """
    def __init__(self, seq_record):
        """
        Holds a sequence from a blast hit and information about it from one or more query alignments. Hit sequence
        information is stored in the following attributes:
            - seq_record        - A protein sequence (Bio.SeqRecord object)
            - hit_pdb_id        - pdb_id of the blast hit (string)
            - hit_chain_id      - chain_id of the blast hit (int)
            - hit_sequence_id   - the sequence id of the blast hit (int)
            - blast_hit         - True if the chain/sequence was returned from blast, false otherwise (boolean)
        Information about the alignment with a particular query is stored in a QueryAlignment object, which are stored
        in the following:
            - query_alignments  - A list of QueryAlignment objects (list)
        """
        self.seq_record = seq_record
        self.hit_pdb_id = None
        self.hit_chain_id = None
        self.hit_sequence_id = None
        self.blast_hit = None
        self.query_alignments = []

    def set_query_alignment(self, record, alignment):
        """
        returns True if a QueryAlignment object was successfully create, otherwise returns False
        """
        logger.debug('In seq_query_alignment()')
        try:
            qa = QueryAlignment()
            pdb_id, chain_id = record.query.encode('ascii').split()[0].split('_')
            query_length = record.query_length
            qa.query_pdb_id = pdb_id
            qa.query_chain_id = chain_id
            qa.query_length = query_length
            qa.alignment = alignment
            qa.set_coverage_and_identity()
            self.query_alignments.append(qa)
            return True
        except:
            logger.exception('Caught exception')
            return False

    def sort_by_coverage(self):
        self.query_alignments.sort(key=lambda qa: qa.coverage, reverse = True)

    def sort_by_identity(self):
        self.query_alignments.sort(key=lambda qa: qa.identity, reverse = True)


class QueryAlignment(object):
    """

    """
    def __init__(self):
        """
        Stores information about the alignment between a blast hit and a particular Query.
            - query_pdb_id      - pdb_id of the query (string)
            - query_chain_id    - chain_id of the query (string)
            - identity          - Percent identity (float)
            - coverage          - Percent coverage (float)
            - alignment         - An BLAST alignment (Bio.Record Alignment object)
            - query_length      - The length of the query sequence used to generate the input alignment (int)
        """
        self.query_id = None
        self.query_pdb_id = None
        self.query_chain_id = None
        self.identity = None
        self.coverage = None
        self.alignment = None
        self.query_length = None

    def set_coverage_and_identity(self):
        """
        Determines the coverage and identity from the alignment and query length attributes and sets these attributes.
        Because these two attributes are prerequisites of the calculation, they must be set before the method call:
            d3r.blast.TestSequence(seq_record)
            seq.alignment = alignment
            seq.query_length
            seq.set_coverage_and_identity()
        If the calculation is successfully carried out, and the attributes successfully assigned, True is returned,
        otherwise, False is returned.
        :return: Boolean
        :return: Boolean
        """
        if not self.query_length or not self.alignment:
            logger.error("The TestSequence instance attributes, " +
                         "Ligand.query_length and Ligand.alignment " +
                         "both must be set " +
                         "before calling the set_coverage_and_identity method.")
            return False
        else:
            coverage, identity = self._calculate_coverage_and_identity(self.alignment, self.query_length)
            self.coverage = coverage
            self.identity = identity
            return True

    def _calculate_coverage_and_identity(self, alignment, query_length):
        """
        Calculates percent identity and percent coverage as fractions that are returned as floats. A input sequence to
        BLASTP is called a 'query' and a returned sequence is called a 'hit' sequence. Using these definitions, the
        percent identity is defined as the fraction of aligned hit residues identical to the query sequence, i.e.
        number_identical / hit_length. The percent coverage is defined as the ratio of the aligned hit length to the
        length of the query sequence, i.e. aligned_hit_length / query_length.
        :param alignment:
        :param record:
        :return: (float) coverage, (float) identity
        """
        logger.debug('In _calculate_coverage_and_identity()')
        aligned_test_seq_length = 0
        identical_test_seq_length = 0
        for hsp in alignment.hsps:
            aligned_test_seq_length += hsp.align_length
            identical_test_seq_length += hsp.identities
        coverage = float(aligned_test_seq_length) / query_length
        identity = float(identical_test_seq_length) / aligned_test_seq_length
        return coverage, identity


#!/usr/bin/env python

__author__ = 'j5wagner'

import commands
import os
import shutil
import re
import glob
import logging
import time
from d3r.celppade.customdock import Dock

logger = logging.getLogger()
logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level   = logging.INFO )



class FredDock(Dock):
    """Implements Fred docking
    """

    SCI_PREPPED_LIG_SUFFIX = '_prepared.sdf'
    SCI_PREPPED_PROT_SUFFIX = '_prepared.pdb'

    def receptor_technical_prep(self, sci_prepped_receptor, pocket_center):
        # "Technical preparation" is the step immediately preceding
        # docking. During this step, you should perform any file
        # conversions or processing that are specific to your docking
        # program.

        # For FRED, receptor_technical_prep takes a scientifically
        # prepared pdb file and creates a docking grid.

        receptor_prefix = sci_prepped_receptor.replace('.pdb','')
        docking_box_file = receptor_prefix + '.oeb.gz'

        # It helps to capture stdout and stderr from these steps for debugging
        stdout_file = receptor_prefix + '_docking_box_gen_stdout'
        stderr_file = receptor_prefix + '_docking_box_gen_stderr'

        # Technical prep for FRED consists of generating a docking box. The
        # box is defined by a .xyz file with two atoms specifying the
        # corners
        box_xyz_file = 'box.xyz'
        center_list = pocket_center.split(',')
        box_size = 30.
        corner_1 = (float(center_list[0]) - (box_size / 2),
                    float(center_list[1]) - (box_size / 2),
                    float(center_list[2]) - (box_size / 2))
        corner_2 = (float(center_list[0]) + (box_size / 2),
                    float(center_list[1]) + (box_size / 2),
                    float(center_list[2]) + (box_size / 2))
        box_xyz_text = '''2 
    CELPP auto-generated docking box 
     X %i %i %i 
     X %i %i %i 
    ''' %(corner_1[0], corner_1[1], corner_1[2], 
          corner_2[0], corner_2[1], corner_2[2])
        with open(box_xyz_file, 'wb') as of:
            of.write(box_xyz_text)

        # Run the box generation command
        cmd = 'receptor_setup -protein ' + sci_prepped_receptor + ' -box ' + box_xyz_file + ' -receptor ' + docking_box_file + ' 2> ' + stderr_file + ' 1> ' + stdout_file
        logging.info('Running tech prep command [%s]' %(cmd))
        commands.getoutput(cmd)

        # Check for succeess
        if not(os.path.exists(docking_box_file)):
            # This step will be run on all ligands, so it helps to be very descriptive about file locations
            abs_stdout_file = os.path.abspath(stdout_file)
            abs_stderr_file = os.path.abspath(stderr_file)
            logging.info('Docking box generation failed. See %s and %s for details' %(abs_stdout_file, abs_stderr_file))
            # CELPPade handles errors by returning False
            return False

        # Finally, we return the filenames that will be needed in the
        # docking step. This list is passed to the dock() function as the
        # tech_prepped_receptor_list argument. Here we pass the docking
        # box file (for the docking) and the original scientifically
        # prepped ligand pdb, as that's the easiest way to return the
        # final receptor conformation.
        return [docking_box_file, sci_prepped_receptor]

    


    def dock(self, tech_prepped_lig_list, tech_prepped_receptor_list, output_receptor_pdb, output_lig_mol):
        # The dock step runs the actual docking algorithm. Its first two
        # arguments are the return values from the technical preparation
        # functions for the ligand and receptor. The outputs from this
        # step must be two files - a pdb with the filename specified in
        # the output_receptor_pdb argument, and a mol with the filename
        # specified in the output_ligand_mol argument.

        tech_prepped_grid = tech_prepped_receptor_list[0]
        sci_prepped_receptor = tech_prepped_receptor_list[1]
        receptor_prefix = os.path.basename(tech_prepped_grid).replace('.oeb.gz','')

        tech_prepped_lig = tech_prepped_lig_list[0]

        # FRED can only output .oeb and .sdf files. We'll have it output a
        # .sdf, then convert it to mol after FRED runs.
        docked_lig_sdf = receptor_prefix + '_docked.sdf'
        #docked_lig_mol = receptor_prefix + '_docked.mol'

        # It helps to capture stdout and stderr from these steps for debugging
        fred_stdout_file = receptor_prefix + '_fred_dock_stdout'
        fred_stderr_file = receptor_prefix + '_fred_dock_stderr'
        babel_stdout_file = receptor_prefix + '_babel_sdf2mol_stdout'
        babel_stderr_file = receptor_prefix + '_babel_sdf2mol_stderr'


        # Run the docking command
        dock_command = 'fred -receptor ' + tech_prepped_grid + ' -dbase ' + tech_prepped_lig + ' -docked_molecule_file ' + docked_lig_sdf + ' 2> ' + fred_stderr_file + ' 1> ' + fred_stdout_file
        logging.info('Running docking command [%s]' %(dock_command))
        commands.getoutput(dock_command)
        if not(os.path.isfile(docked_lig_sdf)):
            logging.info("Docking failed for protein %s and ligand %s" %(tech_prepped_grid, tech_prepped_lig))
            return False

        ## Create the required output files
        # First, the ligand needs to be converted from .sdf to .mol
        lig_convert_command = 'babel -isdf ' + docked_lig_sdf + ' -omol ' + output_lig_mol +  ' 2> ' + babel_stderr_file + ' 1> ' + babel_stdout_file
        commands.getoutput(lig_convert_command)
        if not(os.path.isfile(output_lig_mol)):
            logging.info("Babel conversion failed for docked ligand  %s" %(docked_lig_sdf))
            return False

        # And now we need to prepare the final receptor file. Because this
        # docking didn't modify the receptor, we will just copy the
        # original scientifically-prepped receptor pdb and give it the
        # appropriate name.
        shutil.copyfile(sci_prepped_receptor, output_receptor_pdb)

        # And do a final sanity check
        if not(os.path.isfile(output_receptor_pdb)):
            logging.info("File copy failed for receptor %s" %(output_receptor_pdb))
            return False

        return True

################################################################
########### CONTESTANT MODIFICATIONS SHOULD END HERE ###########
################################################################






if ("__main__") == (__name__):
    from argparse import ArgumentParser
    parser = ArgumentParser()
    parser.add_argument("-s", "--sciprepdir", metavar="PATH", help = "PATH where we can find the scientific protein and lig prep output")
    parser.add_argument("-o", "--outdir", metavar = "PATH", help = "PATH where we will put the docking output")
    parser.add_argument("-u", "--update", action = "store_true",  help = "Update the docking result", default = False) 
    logger = logging.getLogger()
    logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level   = logging.INFO )
    opt = parser.parse_args()
    sci_prep_dir = opt.sciprepdir
    dock_dir = opt.outdir
    update = opt.update
    #running under this dir
    abs_running_dir = os.getcwd()
    log_file_path = os.path.join(abs_running_dir, 'final.log')
    log_file_dest = os.path.join(os.path.abspath(dock_dir), 'final.log')
    my_dock = FredDock()
    my_dock.main_dock(sci_prep_dir, dock_dir, update = update)
    #move the final log file to the result dir
    commands.getoutput("mv %s %s"%(log_file_path, log_file_dest))



#!/usr/bin/env python

__author__ = 'robswift'

import argparse
import os
import sys
import xlsxwriter
from io import BytesIO
import logging

logger = logging.getLogger(__name__)

try:
    from rdkit import Chem
    from rdkit.Chem import Draw
except ImportError:
    logger.exception('Unable to import rdkit images will NOT be added to spreadsheet generated by MakeSheet class')



class MakeSheet(object):

    inchi_dict = {}
    inchi_file = None
    @staticmethod
    def set_inchi_dict(compinchi):
        MakeSheet.inchi_file = compinchi
        handle = open(MakeSheet.inchi_file, 'r')
        for line in handle.readlines():
            try:
                words = line.split()
                if words:
                    inchi = words[0]
                    resname = words[1]
                MakeSheet.inchi_dict[format(resname)] = format(inchi)
            except:
                continue
        handle.close()

    def __init__(self):
        self.out_dir = None
        self.image_file = None
        self.content_dict = None
        self.work_book = None
        self.header = ['Resname', 'Molecule']
        self.header_format = None
        self.body_format = None
        self.size = (25, 39.2)

    def make_sheet(self, out_file):
        bk_file = os.path.join(self.out_dir, 'dockable.xlsx')
        self.work_book = xlsxwriter.Workbook(bk_file)
        work_sheet = self.work_book.add_worksheet()
        work_sheet.set_column('A:A', self.size[0])
        work_sheet.set_column('B:B', self.size[1])
        self.set_header_format()
        self.set_body_format()
        self.write_header(work_sheet)
        self.write_data(work_sheet)

    def set_header_format(self):
        self.header_format = self.work_book.add_format()
        self.header_format.set_align('center')
        self.header_format.set_font_size(16)
        self.header_format.set_bold(True)

    def set_body_format(self):
        self.body_format = self.work_book.add_format()
        self.body_format.set_align('center')
        self.body_format.set_align('vcenter')
        self.body_format.set_font_size(14)

    def write_header(self, work_sheet):
        c = 0
        for col in self.header:
            work_sheet.write_string(0, c, col, self.header_format)
            c += 1

    def write_data(self, work_sheet):
        r = 1
        for resname in self.content_dict.keys():
            work_sheet.set_row(r, 215)
            c = 0
            work_sheet.write_string(r, c, resname, self.body_format)
            c += 1
            if self.make_image(resname):
                image_handle = open(self.image_file, 'rb')
                image_data = BytesIO(image_handle.read())
                image_handle.close()
                self.rm_image()
                work_sheet.insert_image(r, c, "f", {'image_data' : image_data, 'x_offset' : 4 ,'y_offset': 8,
                                            'positioning' : 1})

            else:
                work_sheet.write_string(r, c, 'N/A', self.body_format)
            r += 1
        self.work_book.close()

    def make_image(self, resname):
        if not self.image_file:
            self.set_image_file()
        if not MakeSheet.inchi_dict:
            MakeSheet.set_inchi_dict()
        try:
            inchi = MakeSheet.inchi_dict[resname]
            rd_mol = Chem.MolFromInchi(format(inchi), sanitize=True, treatWarningAsError=True)
            Draw.MolToFile(rd_mol, self.image_file, size=(275,275))
            return True
        except:
            return False

    def set_image_file(self):
        self.image_file = os.path.join(self.out_dir, 'tmp_img.png')

    def rm_image(self):
        os.remove(self.image_file)

def read_log(path, compinchi):
    """
    read log file and add resname and inchi to d = dict {'resname': inchi}
    :param path:
    :return: d
    """
    if not MakeSheet.inchi_dict:
        MakeSheet.set_inchi_dict(compinchi)
    f = os.path.join(path, 'blastnfilter.log')
    content_dict = {}
    handle = open(f, 'r')
    for line in handle.readlines():
        if 'Ligand:' in line:
            resname = line.split()[1].split('|')[0]
            if resname not in content_dict.keys():
                try:
                    inchi = MakeSheet.inchi_dict[resname]
                    content_dict[resname] = inchi
                except KeyError:
                    print resname
                    continue
    return content_dict

def cmd_line():
    parser = argparse.ArgumentParser()

    parser.add_argument("dir", help='stage.2.blastnfilter directory')
    parser.add_argument('--compinchi', help='Path to Components-inchi.ich file',
                        required=True)
    return parser.parse_args()

def run():
    parsed_args = cmd_line()
    out_dir = os.path.abspath(parsed_args.dir)
    content_dict = read_log(out_dir, parsed_args.compinchi)
    make_sheet = MakeSheet()
    make_sheet.out_dir = out_dir
    make_sheet.content_dict = content_dict
    make_sheet.make_sheet(out_dir)

if __name__ == "__main__":
    sys.exit(run())


__author__ = 'robswift'

import sys

from d3r.blast.mcss import MCSS
import logging

logger = logging.getLogger(__name__)

try:
    from rdkit import Chem
    from rdkit.Chem import rdFMCS
    from rdkit import DataStructs
    from rdkit.Chem.Fingerprints import FingerprintMols
    from rdkit.Chem import Descriptors
except ImportError:
    logger.exception('Unable to import rdkit Ligand class will not work')


class Ligand(object):
    """

    """
    # timeout in seconds for rdkit to find maximum common substructure
    FINDMCS_TIMEOUT = 60
    inchi_component = {}

    @staticmethod
    def set_inchi_component(compinchi):
        """
        Each ligand resname is mapped to a InChi string. This information is stored in a class-wide dictionary
        inchi_component with the following structure
        inchi_component = { 'resname' : 'inchi' },
        where 'resname' is the resname used by the PDB for that ligand and 'inchi' is the corresponding PDB InChi
        string.
        :param compinchi: path to the wwPDB file Components-inchi.ich
        """
        try:
            handle = open(compinchi, 'r')
            for line in handle.readlines():
                try:
                    words = line.split()
                    inchi = words[0]
                    resname = words[1]
                    Ligand.inchi_component[format(resname)] = format(inchi)
                except:
                    logger.exception('Caught exception trying to read ' + str(compinchi) + ' file')
                    continue
            handle.close()
        except IOError:
            logger.exception('Caught exception attempting to open or parse Components.inchi.ich file')
            sys.exit(1)
        if len(Ligand.inchi_component.keys()) < 20000:
            raise IOError('There is a problem with Components-inchi.ich.')

    def __init__(self, resname=None, inchi=None):
        self.resname = resname
        self.inchi = inchi
        #modified by sliu 08/04, here to add chain info for ligand 
        self.chain = None 
        self.label = None
        self.rd_mol = None              # a rd mol object that represents
        #modified by sliu 08/08 to add info for the ligand size
        self.size = None 
        self.heavy = None
        self.rot = None
        self.mcsss = []                 # A list of d3r.blast.Mcss objects
        #self.tanimoto = []              # A list of (tanimoto_score, reference_ligand_name)
        self.tanimoto = {}              # A dic of reference_ligand_name:tanimoto_score
        self.mcss_error = None          # set to True, False or None

    def set_rd_mol_from_inchi(self, inchi = None):
        """
        Creates a rdkit.Chem.rdmol Mol object from an input InChI and sets it to self.rd_mol. Hydrogen atoms are not
        removed on successful conversion. If the conversion is unsuccessful, False is returned. If the conversion is
        successful, True is returned.
        :param inchi: (string) An InChI string
        :return: Boolean
        """
        if not inchi and not self.inchi:
            logger.error('No inchi string set')
            return False
        elif not inchi and self.inchi:
            inchi = self.inchi
        try:
            self.rd_mol = Chem.MolFromInchi(format(inchi), removeHs=False, sanitize=False, treatWarningAsError=True)
            return True
        except:
            logger.exception('Unable to create rdkt.Chem.rdmol Mol object')
            return False

    def set_rd_mol_from_resname(self, resname):
        """
        Creates a rdkit.Chem.rdmol from an input resname. If the rdmol is successfully created, the Ligand instance
        attribute, inchi is also set, and True is returned. Otherwise False is returned.
        :param resname: a wwPDB ID (string)
        :return: Boolean
        """
        try:
            inchi = Ligand.inchi_component[resname]
            self.rd_mol = Chem.MolFromInchi(format(inchi), removeHs=False, sanitize=False, treatWarningAsError=True)
            self.inchi = inchi
            return True
        except:
            logger.exception('Unable to create rdkit.Chem.rdmol from resname')
            return False
    
    def set_heavy_size (self,):
        """
        Get number of heavy atoms from the rd_mol object associated with this ligand
        """
        if not self.rd_mol:
            print("Please set an rd mol object before calculating num of heavy atoms")
            logger.error('No rd mol set')
            return False
        try: 
            self.heavy = self.rd_mol.GetNumHeavyAtoms()
            return True
        except:
            logger.exception('Unable to get num of heavy atoms from rd mol object')
            return False

    def set_size (self,):
        """
        Get number of atoms from the rd_mol object associated with this ligand
        """
        if not self.rd_mol:
            print("Please set an rd mol object before calculating num of atoms")
            logger.error('No rd mol set')
            return False
        try:
            self.size = len(self.rd_mol.GetAtoms())
            return True
        except:
            logger.exception('Unable to get num of heavy atoms from rd mol object')
            return False

    def set_rot(self,):
        """
        set number of rotatable bond
        """
        if not self.rd_mol:
            print("Please set an rd mol object before calculating num of atoms")
            logger.error('No rd mol set')
            return False
        try:
            self.rot = Descriptors.NumRotatableBonds(self.rd_mol)
            return True
        except:
            logger.exception('Unable to set the num of rotatable bond from rd mol object')
            return False
    def mcss(self, reference):
        """
        Determines the maximum common substructure (MCSS) between the input, or reference ligand and itself. The MCSS is
        stored in an RDkit mol object, which is returned. If no MCSS was found, the returned mol object will be empty.
        If an error prevents the MCSS calculation from completing, None is returned.
        Before this method is called, the ligand must have a rd_mol attribute set. For example
            lig = Ligand()
            lig.set_rd_mol_from_resname(resname) # where res is a wwPDB ID string
            mcss_mol = lig.mcss(reference)
            lig.set_mcss(reference.resname, mcss_mol)
        :param reference: d3r.blast.Ligand object with the rd_mol attribute set
        :return: the MCSS (rd mol object) or None
        """
        try:
            logger.debug('Trying to find MCS')

            res = rdFMCS.FindMCS([reference.rd_mol, self.rd_mol], timeout=Ligand.FINDMCS_TIMEOUT)
            mcss = Chem.MolFromSmarts(res.smartsString)
            return mcss
        except:
            logger.exception('Caught exception attempting to run rdkit FindMCS or MolFromSmarts')
            return None

    def calc_tanimoto(self, reference):
        """
        Determin the tanimoto similarity score based on the rd mol fingureprint
        """
        try:
            fps_ref = FingerprintMols.FingerprintMol(reference.rd_mol)
            fps_self = FingerprintMols.FingerprintMol(self.rd_mol)
            tanimoto = DataStructs.FingerprintSimilarity(fps_ref, fps_self)
            return tanimoto
        except:
            logger.exception('Caught exception attempting to run rdkit FingerprintMols.FingerprintMol or DataStructs.FingerprintSimilarity')
            return None
    def set_mcss(self, reference, mcss_mol, tanimoto_score):
        """
        Creates an Mcss object and appends it to the list, Ligand.mcsss. It's assumed that the input maximum common
        substructure was determined using the input reference ligand and the ligand object to which the method belongs.
        :param ref_name: d3r.blast.ligand object with the rd_mol attribute set
        :param mcss_mol: the maximum common substructure (rd mol object)
        """
        logger.debug('set_mcss()')
        mcss = MCSS(reference.resname, mcss_mol)
        #here mcss object is stored the mcss rd mol returned from mcss fucntion 
        mcss.set_size()
        #mcss.set_heavy_size()
        mcss.test = self.resname
        mcss.tanimoto = tanimoto_score
        self.mcsss.append(mcss)
    def set_tanimoto(self, reference, tanimoto_score):
        """
        store the tanimoto score and corresponding reference ligand resname
        """
        #self.tanimoto.append((tanimoto_score, reference.resname))
        self.tanimoto[reference.resname] = tanimoto_score



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'churas'

import os.path
import tempfile
import unittest
import shutil
import gzip
import itertools
import string

"""
test_makeblastdb
--------------------------------

Tests for `makeblastdb` module.
"""

from d3r.celpp.task import D3RParameters
from d3r.celpp.task import D3RTask
from d3r.celpp.makeblastdb import MakeBlastDBTask
from tests.celpp import test_task


class TestMakeBlastDBTask(unittest.TestCase):
    def setUp(self):
        pass

    def test_constructor(self):
        params = D3RParameters()
        task = MakeBlastDBTask('/foo', params)
        self.assertEqual(task.get_name(), 'makeblastdb')
        self.assertEqual(task.get_stage(), 1)
        self.assertEqual(task.get_status(), D3RTask.UNKNOWN_STATUS)
        self.assertEqual(task.get_path(), '/foo')
        self.assertEqual(task.get_dir_name(), 'stage.1.makeblastdb')
        test_task.try_update_status_from_filesystem(self, task)

    def test_get_uploadable_files(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = MakeBlastDBTask(temp_dir, params)
            task.create_dir()

            # test on empty dir
            self.assertEqual(task.get_uploadable_files(), [])

            # test with stderr/stdout files
            stdout = os.path.join(task.get_dir(), 'makeblastdb.stdout')
            open(stdout, 'a').close()
            stderr = os.path.join(task.get_dir(), 'makeblastdb.stderr')
            open(stderr, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 2)
            flist.index(stdout)
            flist.index(stderr)

            # test with pdb_seqres.txt.gz file
            open(task.get_pdb_seqres_txt_gz(), 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 3)
            flist.index(stdout)
            flist.index(stderr)
            flist.index(task.get_pdb_seqres_txt_gz())

            os.unlink(task.get_pdb_seqres_txt_gz())

            # test where pdb_seqres.txt is a dir unlikely but why not check
            os.makedirs(task.get_pdb_seqres_txt_gz())
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 2)
            flist.index(stdout)
            flist.index(stderr)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_pdbseqres_txt_gz(self):
        params = D3RParameters()
        task = MakeBlastDBTask('/foo', params)
        self.assertEqual(task.get_pdb_seqres_txt_gz(),
                         os.path.join('/foo', task.get_dir_name(),
                                      'pdb_seqres.txt.gz'))

    def test_get_pdbseqres_txt(self):
        params = D3RParameters()
        task = MakeBlastDBTask('/foo', params)
        self.assertEqual(task.get_pdb_seqres_txt(),
                         os.path.join('/foo', task.get_dir_name(),
                                      'pdb_seqres.txt'))

    def test_get_sequence_count_no_file(self):
        params = D3RParameters()
        task = MakeBlastDBTask('/foo', params)

        self.assertEqual(task._get_sequence_count_message(),
                         '# sequence(s): Error unable to parse file')

    def test_get_sequence_count_file_has_zero_size(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = MakeBlastDBTask(temp_dir, params)
            task.create_dir()
            open(task.get_pdb_seqres_txt(), 'a').close()
            self.assertEqual(task._get_sequence_count_message(),
                             '# sequence(s): 0')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_sequence_count_file_has_no_seqs(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = MakeBlastDBTask(temp_dir, params)
            task.create_dir()
            f = open(task.get_pdb_seqres_txt(), 'w')
            f.write('hi\nhow\nare\n')
            f.flush()
            f.close()
            self.assertEqual(task._get_sequence_count_message(),
                             '# sequence(s): 0')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_sequence_count_file_has_one_seq(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = MakeBlastDBTask(temp_dir, params)
            task.create_dir()
            f = open(task.get_pdb_seqres_txt(), 'w')
            f.write('hi\n>aseq\nare\n')
            f.flush()
            f.close()
            self.assertEqual(task._get_sequence_count_message(),
                             '# sequence(s): 1')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_sequence_count_file_has_multiple_seqs(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = MakeBlastDBTask(temp_dir, params)
            task.create_dir()
            f = open(task.get_pdb_seqres_txt(), 'w')
            f.write('>hi\n>seq\n>are\n')
            f.flush()
            f.close()
            self.assertEqual(task._get_sequence_count_message(),
                             '# sequence(s): 3')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_sequence_count_file_no_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = MakeBlastDBTask(temp_dir, params)
            task.create_dir()
            pdbid = task.get_set_of_pbdid_from_pdb_seqres_txt()
            self.assertEqual(len(pdbid), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pbdid_from_pdb_seqres_txt_no_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = MakeBlastDBTask(temp_dir, params)
            task.create_dir()
            pdbset = task.get_set_of_pbdid_from_pdb_seqres_txt()
            self.assertEqual(len(pdbset), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pbdid_from_pdb_seqres_txt_empty_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = MakeBlastDBTask(temp_dir, params)
            task.create_dir()
            open(task.get_pdb_seqres_txt(), 'a').close()
            self.assertEqual(os.path.isfile(task.get_pdb_seqres_txt()), True)
            pdbset = task.get_set_of_pbdid_from_pdb_seqres_txt()
            self.assertEqual(len(pdbset), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pbdid_from_pdb_seqres_txt_file_no_seqs(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = MakeBlastDBTask(temp_dir, params)
            task.create_dir()
            f = open(task.get_pdb_seqres_txt(), 'w')
            f.write('hi\nhow\nare\nyou')
            f.flush()
            f.close()
            self.assertEqual(os.path.isfile(task.get_pdb_seqres_txt()), True)
            pdbset = task.get_set_of_pbdid_from_pdb_seqres_txt()
            self.assertEqual(len(pdbset), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pbdid_from_pdb_seqres_txt_with_seqs(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = MakeBlastDBTask(temp_dir, params)
            task.create_dir()
            f = open(task.get_pdb_seqres_txt(), 'w')
            f.write('>101m_A mol:protein length:154  MYOGLOBIN\n')
            f.write('MVLSEGEWQLVLHVWAKVEADVAGHGQDILIRLFKSHPETLEKFDRVK'
                    'HLKTEAEMKASEDLKKHG\n')
            f.write('>102l_A mol:protein length:165  T4 LYSOZYME\n')
            f.write('MNIFEMLRIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNAAAKSELDKA'
                    'IGRNTNGVITKDEAEKLFNQDVDAAVRGILRNAKLKPVYDSLDAVRRAAL'
                    'INMVFQMGETGVAGFTNSLRMLQQKRWDEAAVNLAKSRWYNQTPNRAKRV'
                    'ITTFRTGTWDAYKNL\n')
            f.write('>102l_A mol:protein length:154  MYOGLOBIN\n')
            f.write('MVLSEGEWQLVLHVWAKVEADVAGHGQDILIRLFKSHPETLEKFDRFKHL'
                    'KTEAEMKASEDLKKAGVTVLTALGAILKKKGHHEAELKPLAQSHATKHKI'
                    'PIKYLEFISEAIIHVLHSRHPGNFGADAQGAMNKALELFRKDIAAKYKELGYQG\n')
            f.write('>103l_A mol:protein length:167  T4 LYSOZYME\n')
            f.write('MNIFEMLRIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNSLDAAKSELD'
                    'KAIGRNTNGVITKDEAEKLFNQDVDAAVRGILRNAKLKPVYDSLDAVRRA'
                    'ALINMVFQMGETGVAGFTNSLRMLQQKRWDEAAVNLAKSRWYNQTPNRAK'
                    'RVITTFRTGTWDAYKNL\n')
            f.write('>10jj3m_A mol:protein length:154  MYOGLOBIN\n')
            f.write('MVLSEGEWQLVLHVWAKVEADVAGHGQDILIRLFKSHPETLEKFDRF\n')
            f.write('>104l_A mol:protein length:166  T4 LYSOZYME\n')
            f.write('MNIFEMLRIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNAAKSAAE\n')
            f.write('>104l_B mol:protein length:166  T4 LYSOZYME\n')
            f.write('MNIFEMLRIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNAAKSAKNL\n')

            f.flush()
            f.close()
            self.assertEqual(os.path.isfile(task.get_pdb_seqres_txt()), True)
            pdbset = task.get_set_of_pbdid_from_pdb_seqres_txt()
            self.assertEqual(len(pdbset), 4)
            self.assertEqual('101M' in pdbset, True)
            self.assertEqual('102L' in pdbset, True)
            self.assertEqual('103L' in pdbset, True)
            self.assertEqual('104L' in pdbset, True)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pbdid_from_pdb_seqres_txt_wrong_len_pdbids(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = MakeBlastDBTask(temp_dir, params)
            task.create_dir()
            f = open(task.get_pdb_seqres_txt(), 'w')
            f.write('>1m_A mol:protein length:154  MYOGLOBIN\n')
            f.write('MVLSEGEWQLVLHVWAKVEADVAGHGQDILIRLFKSHPETLEKFDRVK'
                    'HLKTEAEMKASEDLKKHG\n')
            f.write('>abcdel_A mol:protein length:165  T4 LYSOZYME\n')
            f.write('MNIFEMLRIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNAAAKSELDKA'
                    'IGRNTNGVITKDEAEKLFNQDVDAAVRGILRNAKLKPVYDSLDAVRRAAL'
                    'INMVFQMGETGVAGFTNSLRMLQQKRWDEAAVNLAKSRWYNQTPNRAKRV'
                    'ITTFRTGTWDAYKNL\n')

            f.flush()
            f.close()
            self.assertEqual(os.path.isfile(task.get_pdb_seqres_txt()), True)
            pdbset = task.get_set_of_pbdid_from_pdb_seqres_txt()
            self.assertEqual(len(pdbset), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pbdid_from_pdb_seqres_txt_with_400k_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = MakeBlastDBTask(temp_dir, params)
            task.create_dir()
            f = open(task.get_pdb_seqres_txt(), 'w')
            perms = itertools.permutations(string.ascii_lowercase +
                                           '123456789', 4)
            counter = 0
            try:
                limit = 400000
                while counter < limit:
                    f.write('>' + ''.join(map(str, perms.next())) +
                            '_A mol:protein length:165  T4 LYSOZYME\n')
                    f.write('MVLSEGEWQLVLH\n')
                    counter += 1
            except StopIteration:
                pass
            f.flush()
            f.close()

            pdbset = task.get_set_of_pbdid_from_pdb_seqres_txt()
            self.assertEqual(len(pdbset), counter)

        finally:
            shutil.rmtree(temp_dir)

    def test_can_run_where_task_is_complete(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = MakeBlastDBTask(temp_dir, params)
            task.create_dir()
            open(os.path.join(task.get_dir(), 'complete'), 'a').close()
            self.assertEqual(task.can_run(), False)
        finally:
            shutil.rmtree(temp_dir)

    def test_can_run_where_task_failed(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = MakeBlastDBTask(temp_dir, params)
            task.create_dir()
            open(os.path.join(task.get_dir(), 'error'), 'a').close()
            self.assertEqual(task.can_run(), False)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_where_can_run_is_false(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = MakeBlastDBTask(temp_dir, params)
            task.create_dir()
            open(os.path.join(task.get_dir(), 'error'), 'a').close()
            task.run()

            self.assertEqual(task._can_run, False)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_where_pdbsequrl_is_not_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = MakeBlastDBTask(temp_dir, params)
            task.run()
            self.assertEqual(task.get_error(), 'cannot download files cause '
                                               'pdbsequrl not set')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_where_makeblastdb_is_not_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.pdbsequrl = 'pdbsequrl'
            task = MakeBlastDBTask(temp_dir, params)
            task.run()
            self.assertEqual(task.get_error(), 'cannot make blast database '
                                               'cause makeblastdb not set')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_where_download_fails(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.pdbsequrl = 'file://doesnotexist'
            params.makeblastdb = 'makeblastdb'
            task = MakeBlastDBTask(temp_dir, params)
            task._retrysleep = 0
            task._maxretries = 1
            task.run()
            self.assertEqual(task.get_error(), 'Unable to download file: ' +
                             'file://doesnotexist')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_where_gunzip_fails(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            fakegz = os.path.join(temp_dir, 'fake.gz')

            f = open(fakegz, 'w')
            f.write('hello\n')
            f.flush()
            f.close()

            params.pdbsequrl = 'file://'+fakegz
            params.makeblastdb = 'makeblastdb'
            task = MakeBlastDBTask(temp_dir, params)
            task._retrysleep = 0
            task._maxretries = 1
            task.run()
            self.assertEqual(task.get_error(), 'Unable to uncompress file: ' +
                             task.get_pdb_seqres_txt())
        finally:
            shutil.rmtree(temp_dir)

    def test_run_where_makeblastdb_fails(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            fakegz = os.path.join(temp_dir, 'fake.gz')

            f = gzip.open(fakegz, 'wb')
            f.write('hello\n')
            f.flush()
            f.close()

            params.pdbsequrl = 'file://'+fakegz
            params.makeblastdb = 'false'
            task = MakeBlastDBTask(temp_dir, params)
            task._retrysleep = 0
            task._maxretries = 1
            task.run()
            self.assertEqual(task.get_error(), 'Non zero exit code: 1 '
                                               'received. Standard out:'
                                               '  Standard error: ')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_where_everything_is_successful(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            fakegz = os.path.join(temp_dir, 'fake.gz')

            f = gzip.open(fakegz, 'wb')
            f.write('hello\n')
            f.flush()
            f.close()

            params.pdbsequrl = 'file://'+fakegz
            params.makeblastdb = 'echo'
            task = MakeBlastDBTask(temp_dir, params)
            task._retrysleep = 0
            task._maxretries = 1
            task.run()
            self.assertEqual(task.get_error(), None)

            # check echo.stdout file for valid arguments
            f = open(os.path.join(task.get_dir(), 'echo.stdout'), 'r')
            line = f.readline()

            self.assertEqual(line, '-in ' +
                             task.get_pdb_seqres_txt() +
                             ' -out ' +
                             os.path.join(task.get_dir(), 'pdb_db') +
                             ' -dbtype prot\n')

            f.close()

            lines = task.get_email_log().split('\n')
            self.assertEqual(lines[2], '# sequence(s): 0')
            f.close()
        finally:
            shutil.rmtree(temp_dir)

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

__author__ = 'j5wagner'

import commands
import sys
import os
import glob
import logging
import time
import re 
import shutil
from d3r.utilities.challenge_data import ChallengeData
from d3r.utilities.readers import ReadText

logger = logging.getLogger(__name__)


class LigandPrep(object):
    """Abstract class defining methods for a custom protein and ligand preparation solution
    for CELPP
    """

    # This prep script will be required to output files with the appropriate suffixes
    OUTPUT_LIG_SUFFIX = '.smi'
    

    def ligand_scientific_prep(self, 
                               lig_smi_file, 
                               out_lig_file, 
                               targ_info_dict={}):
       """Does not do any scientific preparation - Passes ligand smiles file forward without any processing 
       """
       
       shutil.copyfile(lig_smi_file, out_lig_file)
       return True
    

    def run_scientific_ligand_prep(self, challenge_data_path, pdb_protein_path, working_folder):
        abs_challenge_data_path = os.path.abspath(challenge_data_path)
        chal_data_obj = ChallengeData(abs_challenge_data_path)
        if not(chal_data_obj.is_valid_for_celpp()):
            logging.info('%s is not a valid CELPP challenge data directory. Unable to run ligand prep.' %(abs_challenge_data_path))
            return False
        week_chal_data_dict = chal_data_obj.get_targets()
        week_name = week_chal_data_dict.keys()[0]
        abs_week_path = os.path.join(abs_challenge_data_path, week_name)
        pot_target_dirs = week_chal_data_dict[week_name]
        os.chdir(working_folder)
        current_dir_layer_1 = os.getcwd()

        ## Get all potential target directories and candidates within
        valid_targets = {}

        # Ensure that the challengedata targets are valid and copy in files
        for pot_target_dir in pot_target_dirs:
            os.chdir(current_dir_layer_1)
            pot_targ_id = os.path.basename(pot_target_dir.strip('/'))

            # Does it look like a pdb id?
            if len(pot_targ_id) != 4:
                logging.info('Filtering potential target directories: %s is not 4 characters long. Skipping' %(pot_targ_id))
                continue
            os.mkdir(pot_targ_id)
            target_dir_path = os.path.join(abs_week_path, pot_targ_id)

            # Copy in <targ id>.txt file
            targ_info_basename = pot_targ_id + '.txt'
            origin_txt_file = os.path.join(target_dir_path, targ_info_basename)
            dest_txt_file = os.path.join(pot_targ_id, targ_info_basename)
            shutil.copyfile(origin_txt_file, dest_txt_file)


            # Pull in the ligand inchi/smiles
            lig_smiles_files = glob.glob('%s/lig_*.smi' %(target_dir_path))
            if len(lig_smiles_files) != 1:
                logging.info('Unable to find unambiguous ligand smiles for %s - glob returned %r' %(pot_targ_id, lig_smiles_files))
                continue
            lig_smiles_file = lig_smiles_files[0]
            local_smiles_file = os.path.basename(lig_smiles_file)
            dest_smiles_file = os.path.join(pot_targ_id, local_smiles_file)
            shutil.copyfile(lig_smiles_file, dest_smiles_file)

            valid_targets[pot_targ_id] = local_smiles_file

        for target_id in valid_targets.keys():
            os.chdir(current_dir_layer_1)
            os.chdir(target_id)
            smiles_filename = valid_targets[target_id]

            # Parse the <targ id>.txt file
            ReadText_obj = ReadText()
            targ_info_dict = ReadText_obj.parse_txt(target_id + '.txt')

            # Prepare the ligand
            lig_prefix = smiles_filename.replace('.smi','')
            prepared_lig_file = '%s_prepared%s' %(lig_prefix, LigandPrep.OUTPUT_LIG_SUFFIX)
            try:
                lig_prep_result = self.ligand_scientific_prep(smiles_filename, 
                                                              prepared_lig_file,
                                                              targ_info_dict=targ_info_dict)
            except:
                logging.info(sys.exc_info())
                logging.info("try/except caught error in ligand_scientific_prep function.  Skipping target %s" %(target_id))
                continue
            if lig_prep_result == False:
                logging.info("Unable to prepare the ligand for this target protein: %s. Skipping" %(target_id))
                continue 

            if not(os.path.exists(prepared_lig_file)):
                logging.info('Expected output file %s does not exist. Assuming that ligand prep failed. Skipping target %s' %(prepared_lig_file, target_id))
                continue
            if os.path.getsize(prepared_lig_file)==0:
                logging.info('Expected output file %s has size 0. Assuming that ligand prep failed. Skipping candidate %s' %(prepared_lig_file, target_id))
                continue

            logging.info("Successfully prepared ligand %s for target %s"%(lig_prefix, target_id))




#!/usr/bin/env python
# -*- coding: utf-8 -*-


import unittest
import tempfile
import shutil
import os.path
from mock import Mock

from d3r.celpp.filetransfer import FtpFileTransfer

"""
test_uploader
----------------------------------

Tests for `uploader` module.
"""


class MockFtp(object):
    """Dummy ftp connection
    """


class TestFtpFileTransfer(unittest.TestCase):

    def setUp(self):
        pass

    def test_constructor(self):
        # test passing None which means we set everything manually
        foo = FtpFileTransfer(None)

        self.assertEqual(foo.get_host(), None)
        self.assertEqual(foo.get_password(), None)
        self.assertEqual(foo.get_user(), None)
        self.assertEqual(foo.get_remote_dir(), '')
        self.assertEqual(foo.get_error_msg(), None)
        self.assertEqual(foo.get_remote_challenge_dir(), None)

        temp_dir = tempfile.mkdtemp()
        try:
            # test passing non existant config file
            try:
                foo = FtpFileTransfer(os.path.join(temp_dir, 'doesnotexist'))
                self.fail('Expected IOError')
            except IOError:
                pass

            # test passing empty config file
            open(os.path.join(temp_dir, 'empty'), 'a').close()
            foo = FtpFileTransfer(os.path.join(temp_dir, 'empty'))
            self.assertEqual(foo.get_host(), None)
            self.assertEqual(foo.get_password(), None)
            self.assertEqual(foo.get_user(), None)
            self.assertEqual(foo.get_remote_dir(), '')
            self.assertEqual(foo.get_remote_challenge_dir(), None)
            self.assertEqual(foo.get_contestant_id(), None)

            # test getters and setters
            foo.set_connect_timeout(10)
            foo.set_connection('hi')
            foo.set_host('host')
            foo.set_password('pass')
            foo.set_remote_dir('/remote')
            foo.set_user('user')
            foo.set_remote_challenge_dir('/chall')
            foo.set_contestant_id(12345)

            self.assertEqual(foo.get_connect_timeout(), 10)
            self.assertEqual(foo.get_host(), 'host')
            self.assertEqual(foo.get_password(), 'pass')
            self.assertEqual(foo.get_remote_dir(), '/remote')
            self.assertEqual(foo.get_user(), 'user')
            self.assertEqual(foo.get_remote_challenge_dir(), '/chall')
            self.assertEqual(foo.get_contestant_id(), '12345')

            foo.set_contestant_id(None)
            self.assertEqual(foo.get_contestant_id(), None)

        finally:
            shutil.rmtree(temp_dir)

    def test_connect(self):
        # test where alt_ftp_con is None and we use ftpretty
        foo = FtpFileTransfer(None)
        foo.set_host('doesnotexist')
        foo.set_user('user')
        foo.set_password('')
        foo.set_remote_dir('/remote')
        foo.set_connect_timeout(0)
        self.assertFalse(foo.connect())

        # test where alt_ftp_con is set
        foo = FtpFileTransfer(None)
        foo.set_connection('hi')
        foo.connect()
        self.assertEqual(foo._ftp, 'hi')

    def test_disconnect(self):
        # test disconnect where _ftp is None
        foo = FtpFileTransfer(None)
        foo.disconnect()

        # test disconnect where _ftp is set and
        # so is _alt_ftp_con
        foo = FtpFileTransfer(None)
        foo.set_connection('hi')
        foo.connect()
        foo.disconnect()

        # test disconnect where _ftp is set and
        # _alt_ftp_con is None
        foo = FtpFileTransfer(None)
        mockftp = MockFtp()
        mockftp.close = Mock(side_effect=Exception())
        foo.set_connection(mockftp)
        foo.connect()
        foo._alt_ftp_con = None
        foo.disconnect()
        mockftp.close.assert_any_call()

    def test_upload_file_direct_none_passed_as_file(self):
        foo = FtpFileTransfer(None)

        self.assertEqual(foo.upload_file_direct(None, '/remote_dir',
                                                'remote_filename'), False)

        self.assertEqual(foo.get_error_msg(), 'File passed in is None')

    def test_upload_file_direct_file_is_not_a_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            foo = FtpFileTransfer(None)
            noexist = os.path.join(temp_dir, 'noexist')
            self.assertEqual(foo.upload_file_direct(noexist,
                                                    '/remote_dir',
                                                    'remote_filename'), False)

            self.assertEqual(foo.get_error_msg(), noexist +
                             ' is not a file')
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_file_direct_remote_dir_is_none(self):
        temp_dir = tempfile.mkdtemp()
        try:
            foo = FtpFileTransfer(None)
            afile = os.path.join(temp_dir, 'afile')
            open(afile, 'a').close()
            self.assertEqual(foo.upload_file_direct(afile, None,
                                                    'remote_filename'), False)
            self.assertEqual(foo.get_error_msg(), 'remote_dir is None')
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_file_direct_remote_file_name_is_none(self):
        temp_dir = tempfile.mkdtemp()
        try:
            foo = FtpFileTransfer(None)
            afile = os.path.join(temp_dir, 'afile')
            open(afile, 'a').close()
            self.assertEqual(foo.upload_file_direct(afile, '/foo', None),
                             False)
            self.assertEqual(foo.get_error_msg(), 'remote_file_name is None')
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_file_direct_put_throws_exception(self):
        temp_dir = tempfile.mkdtemp()
        try:
            mockftp = MockFtp()
            mockftp.put = Mock(side_effect=IOError('hi'))
            foo = FtpFileTransfer(None)
            foo.set_connection(mockftp)
            foo.connect()
            afile = os.path.join(temp_dir, 'afile')
            open(afile, 'a').close()
            self.assertEqual(foo.upload_file_direct(afile, '/foo', 'name'),
                             False)
            self.assertEqual(foo.get_error_msg(), 'Unable to upload ' + afile +
                             ' to /foo/name : hi')
            foo.disconnect()
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_file_direct_success(self):
        temp_dir = tempfile.mkdtemp()
        try:
            mockftp = MockFtp()
            mockftp.put = Mock(return_value=3)
            foo = FtpFileTransfer(None)
            foo.set_connection(mockftp)
            foo.connect()
            afile = os.path.join(temp_dir, 'afile')
            open(afile, 'a').close()
            self.assertEqual(foo.upload_file_direct(afile, '/foo', 'name'),
                             True)
            self.assertEqual(foo.get_error_msg(), None)
            self.assertEqual(foo.get_upload_summary(), '1 (3 bytes) files '
                                                       'uploaded in 0 seconds'
                                                       ' to host Unset:')
            foo.disconnect()
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_file_where_file_is_none(self):

        # test where file is None
        ftpspec = ["put"]
        mockftp = Mock(spec=ftpspec)
        foo = FtpFileTransfer(None)
        foo.set_connection(mockftp)
        foo.connect()
        foo._upload_file(None)

    def test_upload_file_where_file_does_not_exist(self):
        temp_dir = tempfile.mkdtemp()
        try:
            ftpspec = ["put"]
            mockftp = Mock(spec=ftpspec)
            foo = FtpFileTransfer(None)
            foo.set_connection(mockftp)
            foo.connect()
            foo._upload_file(os.path.join(temp_dir, 'nonexist'))
            self.assertEqual(mockftp.call_count, 0)

        finally:
            shutil.rmtree(temp_dir)

    def test_upload_file_that_raises_exception(self):
        temp_dir = tempfile.mkdtemp()
        try:

            mockftp = MockFtp()
            mockftp.put = Mock(side_effect=IOError('hi'))
            foo = FtpFileTransfer(None)
            foo.set_connection(mockftp)
            foo.connect()
            valid_file = os.path.join(temp_dir, 'file')
            f = open(valid_file, 'a')
            f.write('12')
            f.flush()
            f.close()
            try:
                foo._upload_file(valid_file)
                self.fail('Expected IOError')
            except IOError:
                pass
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_file_on_valid_file(self):

        temp_dir = tempfile.mkdtemp()
        try:
            mockftp = MockFtp()
            mockftp.put = Mock(return_value=3)
            foo = FtpFileTransfer(None)
            foo.set_connection(mockftp)
            foo.set_remote_dir('/remote')
            foo.connect()
            valid_file = os.path.join(temp_dir, 'file')
            f = open(valid_file, 'a')
            f.write('12')
            f.flush()
            f.close()
            foo._upload_file(valid_file)
            mockftp.put.assert_called_with(valid_file,
                                           os.path.normpath('/remote' +
                                                            valid_file))
            self.assertEqual(foo._bytes_transferred, 3)
            self.assertEqual(foo._files_transferred, 1)
            foo._upload_file(valid_file)
            self.assertEqual(foo._bytes_transferred, 6)
            self.assertEqual(foo._files_transferred, 2)
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_files_file_list_is_none(self):
        foo = FtpFileTransfer(None)
        self.assertEqual(foo.upload_files(None), True)
        self.assertEqual(foo.get_error_msg(),
                         'List of files passed in was None')
        self.assertEqual(foo.get_upload_summary(),
                         'List of files passed in was None\n'
                         '0 (0 bytes) files uploaded in 0 '
                         'seconds to host Unset:')

    def test_upload_files_file_list_is_empty(self):
        foo = FtpFileTransfer(None)
        self.assertEqual(foo.upload_files([]), True)
        self.assertEqual(foo.get_error_msg(),
                         'No files to upload')
        self.assertEqual(foo.get_upload_summary(),
                         'No files to upload\n'
                         '0 (0 bytes) files uploaded in 0 '
                         'seconds to host Unset:')

    def test_upload_files_one_valid_file(self):
        mockftp = MockFtp()
        mockftp.put = Mock(return_value=3)
        mockftp.close = Mock(return_value=None)
        foo = FtpFileTransfer(None)
        foo.set_connection(mockftp)

        foo.set_remote_dir('/remote')
        foo.set_host('hosty')
        temp_dir = tempfile.mkdtemp()
        try:
            foo.connect()
            valid_file = os.path.join(temp_dir, 'valid')
            f = open(valid_file, 'a')
            f.write('hi')
            f.flush()
            f.close()

            self.assertEqual(foo.upload_files([valid_file]), True)
            self.assertEqual(foo.get_error_msg(), None)
            self.assertEqual(foo.get_upload_summary(),
                             '1 (3 bytes) files uploaded in 0 '
                             'seconds to host hosty:/remote')
            foo.disconnect()
            mockftp.put.assert_called_with(valid_file,
                                           os.path.normpath('/remote' +
                                                            valid_file))
            mockftp.close.assert_not_called()
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_files_multiple_valid_files(self):
        mockftp = MockFtp()
        mockftp.put = Mock(return_value=3)
        mockftp.close = Mock(return_value=None)
        foo = FtpFileTransfer(None)
        foo.set_connection(mockftp)

        foo.set_remote_dir('/remote')
        foo.set_host('hosty')
        temp_dir = tempfile.mkdtemp()
        try:
            foo.connect()
            valid_file = os.path.join(temp_dir, 'valid')
            f = open(valid_file, 'a')
            f.write('hi')
            f.flush()
            f.close()
            afile = os.path.join(temp_dir, 'hi')
            open(afile, 'a').close()

            self.assertEqual(foo.upload_files([valid_file, afile]), True)
            self.assertEqual(foo.get_error_msg(), None)
            self.assertEqual(foo.get_upload_summary(),
                             '2 (6 bytes) files uploaded in 0 '
                             'seconds to host hosty:/remote')
            foo.disconnect()
            self.assertEqual(mockftp.put.call_count, 2)

            mockftp.close.assert_not_called()
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_files_where_file_upload_raises_exception(self):
        mockftp = MockFtp()
        mockftp.put = Mock(side_effect=IOError('hi'))
        mockftp.close = Mock(return_value=None)
        foo = FtpFileTransfer(None)
        foo.set_connection(mockftp)
        foo.set_remote_dir('/remote')
        foo.set_host('hosty')
        temp_dir = tempfile.mkdtemp()
        try:
            foo.connect()
            valid_file = os.path.join(temp_dir, 'valid')
            f = open(valid_file, 'a')
            f.write('hi')
            f.flush()
            f.close()

            self.assertEqual(foo.upload_files([valid_file]), False)
            self.assertEqual(foo.get_error_msg(), 'Error during upload')
            self.assertEqual(foo.get_upload_summary(),
                             'Error during upload\n'
                             '0 (0 bytes) files uploaded in 0 '
                             'seconds to host hosty:/remote')
            foo.disconnect()
            mockftp.put.assert_called_with(valid_file,
                                           os.path.normpath('/remote' +
                                                            valid_file))
        finally:
            shutil.rmtree(temp_dir)

    def test_get_upload_summary_remotedir_not_set(self):
        foo = FtpFileTransfer(None)
        sum = foo.get_upload_summary()
        self.assertEqual(sum, '0 (0 bytes) files uploaded in 0 seconds to'
                              ' host Unset:')

    def test_delete_file_none_file(self):
        foo = FtpFileTransfer(None)
        self.assertFalse(foo.delete_file(None))
        self.assertEqual(foo.get_error_msg(), 'remote_file None')

    def test_delete_file_not_connected(self):
        foo = FtpFileTransfer(None)
        self.assertFalse(foo.delete_file('/./somefile'))
        self.assertEqual(foo.get_error_msg(), "Unable to delete /somefile : "
                                              "'NoneType' object has no "
                                              "attribute 'delete'")

    def test_delete_file_success(self):
        mockftp = MockFtp()
        mockftp.delete = Mock(return_value='hello')
        mockftp.close = Mock(return_value=None)
        foo = FtpFileTransfer(None)
        foo.set_connection(mockftp)
        foo.connect()
        self.assertTrue(foo.delete_file('/a/b'))
        self.assertEqual(foo.get_error_msg(), None)
        foo.disconnect()
        mockftp.delete.assert_called_with('/a/b')

    def test_delete_file_fail(self):
        mockftp = MockFtp()
        mockftp.delete = Mock(side_effect=IOError('error'))
        mockftp.close = Mock(return_value=None)
        foo = FtpFileTransfer(None)
        foo.set_connection(mockftp)
        foo.connect()
        self.assertFalse(foo.delete_file('/a/b'))
        self.assertEqual(foo.get_error_msg(), 'Unable to delete /a/b : error')
        foo.disconnect()
        mockftp.delete.assert_called_with('/a/b')

    def test_download_file_remote_file_params_none(self):
        foo = FtpFileTransfer(None)
        self.assertFalse(foo.download_file(None, None))
        self.assertEqual(foo.get_error_msg(), 'remote_file None')

        self.assertFalse(foo.download_file('/hi', None))
        self.assertEqual(foo.get_error_msg(), 'local_file None')

    def test_download_file_not_connected(self):
        foo = FtpFileTransfer(None)
        self.assertFalse(foo.download_file('/remote/bye', '/local/hi'))
        self.assertEqual(foo.get_error_msg(), "Unable to download /remote/bye "
                                              "to /local/hi : 'NoneType' "
                                              "object has no attribute 'get'")

    def test_download_file_success(self):
        mockftp = MockFtp()
        mockftp.get = Mock()
        mockftp.close = Mock(return_value=None)
        foo = FtpFileTransfer(None)
        foo.set_connection(mockftp)
        foo.connect()
        self.assertTrue(foo.download_file('/a/b', '/c/d'))
        self.assertEqual(foo.get_error_msg(), None)
        foo.disconnect()
        mockftp.get.assert_called_with('/a/b', local='/c/d')

    def test_download_file_fail(self):
        mockftp = MockFtp()
        mockftp.get = Mock(side_effect=IOError('error'))
        mockftp.close = Mock(return_value=None)
        foo = FtpFileTransfer(None)
        foo.set_connection(mockftp)
        foo.connect()
        self.assertFalse(foo.download_file('/a/b', '/c/d'))
        self.assertEqual(foo.get_error_msg(), 'Unable to download /a/b to '
                                              '/c/d : error')
        foo.disconnect()
        mockftp.get.assert_called_with('/a/b', local='/c/d')

    def test_list_dirs_remote_dir_none(self):
        foo = FtpFileTransfer(None)
        self.assertFalse(foo.list_dirs(None))
        self.assertEqual(foo.get_error_msg(), 'remote_dir None')

    def test_list_dirs_remote_dir_not_connected(self):
        foo = FtpFileTransfer(None)
        self.assertFalse(foo.list_dirs('/foo'))
        self.assertEqual(foo.get_error_msg(), "Unable to get directory list "
                                              "for /foo : 'NoneType' object "
                                              "has no attribute 'list'")

    def test_list_dirs_success(self):
        mockftp = MockFtp()
        mockftp.list = Mock(return_value=[{'directory': 'd', 'name': '.'},
                                          {'directory': 'd', 'name': '..'},
                                          {'directory': 'd', 'name': 'foo'},
                                          {'directory': '-', 'name': 'file'}])
        mockftp.close = Mock(return_value=None)
        foo = FtpFileTransfer(None)
        foo.set_connection(mockftp)
        foo.connect()
        dirlist = foo.list_dirs('/foo')
        self.assertTrue(len(dirlist) == 1)
        self.assertEqual(dirlist[0], 'foo')
        foo.disconnect()
        mockftp.list.assert_called_with('/foo', extra=True)

    def test_list_dirs_fail(self):
        mockftp = MockFtp()
        mockftp.list = Mock(side_effect=IOError('error'))
        mockftp.close = Mock(return_value=None)
        foo = FtpFileTransfer(None)
        foo.set_connection(mockftp)
        foo.connect()
        dirlist = foo.list_dirs('/foo2')
        self.assertTrue(dirlist is None)
        foo.disconnect()
        self.assertEqual(foo.get_error_msg(), 'Unable to get directory list '
                                              'for /foo2 : error')
        mockftp.list.assert_called_with('/foo2', extra=True)

    def test_list_files_remote_dir_none(self):
        foo = FtpFileTransfer(None)
        self.assertFalse(foo.list_files(None))
        self.assertEqual(foo.get_error_msg(), 'remote_dir None')

    def test_list_files_remote_dir_not_connected(self):
        foo = FtpFileTransfer(None)
        self.assertFalse(foo.list_files('/foo'))
        self.assertEqual(foo.get_error_msg(), "Unable to get file list "
                                              "for /foo : 'NoneType' object "
                                              "has no attribute 'list'")

    def test_list_files_success(self):
        mockftp = MockFtp()
        mockftp.list = Mock(return_value=[{'directory': 'd', 'name': '.'},
                                          {'directory': 'd', 'name': '..'},
                                          {'directory': 'd', 'name': 'foo'},
                                          {'directory': '-', 'name': 'file'}])
        mockftp.close = Mock(return_value=None)
        foo = FtpFileTransfer(None)
        foo.set_connection(mockftp)
        foo.connect()
        filelist = foo.list_files('/foo')
        self.assertTrue(len(filelist) == 1)
        self.assertEqual(filelist[0], 'file')
        foo.disconnect()
        mockftp.list.assert_called_with('/foo', extra=True)

    def test_list_files_fail(self):
        mockftp = MockFtp()
        mockftp.list = Mock(side_effect=IOError('error'))
        mockftp.close = Mock(return_value=None)
        foo = FtpFileTransfer(None)
        foo.set_connection(mockftp)
        foo.connect()
        filelist = foo.list_files('/foo2')
        self.assertTrue(filelist is None)
        foo.disconnect()
        self.assertEqual(foo.get_error_msg(), 'Unable to get file list '
                                              'for /foo2 : error')
        mockftp.list.assert_called_with('/foo2', extra=True)

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

__author__ = 'robswift'

import sys
import logging
import argparse

from d3r.utilities import run
import d3r
from d3r.celpp import util

# create logger
logger = logging.getLogger('d3r.blastnfilter')
LOG_FORMAT = "%(asctime)-15s %(levelname)s %(name)s %(message)s"

class CommandLineParameters(object):
    """Holds command line arguments
    """
    pass


def _parse_arguments(desc, args):
    """Parses command line arguments using argparse.
    """
    parsed_arguments = CommandLineParameters()

    help_formatter = argparse.RawDescriptionHelpFormatter
    parser = argparse.ArgumentParser(description=desc,
                                     formatter_class=help_formatter)
    parser.add_argument("--outdir", required=True, dest="out",
                        help='Directory to write output to ')
    parser.add_argument("--nonpolymertsv", required=True,
                        dest="non_polymer",
                        help='Nonpolymer tsv file ')
    parser.add_argument("--sequencetsv", dest="polymer",
                        required=True,
                        help='Sequence tsv file ')
    parser.add_argument("--crystalpH", dest="ph",
                        required=True,
                        help='Crystal ph file')
    parser.add_argument("--pdbblastdb", dest="blast_db",
                        required=True,
                        help='Blast database directory')
    parser.add_argument("--pdbdb", dest="pdb_path",
                        required=True,
                        help='PDB Database directory')
    parser.add_argument("--compinchi", required=True,
                        help='Components.inchi file')
    parser.add_argument("--log", dest="loglevel", choices=['DEBUG',
                        'INFO', 'WARNING', 'ERROR', 'CRITICAL'],
                        help="Set the logging level",
                        default='WARNING')
    parser.add_argument('--version', action='version',
                        version=('%(prog)s ' + d3r.__version__))
    return parser.parse_args(args, namespace=parsed_arguments)

def main(args):
    desc = """
              Blastnfilter
              """
    options = _parse_arguments(desc, args[1:])
    options.program = args[0]
    options.version = d3r.__version__

    util.setup_logging(options)
    logger.debug('Starting run ')
    try:
        run.run(options)
    except Exception:
        logger.exception("Error caught exception")
        sys.exit(2)
    sys.exit(0)

if __name__ == "__main__":
    main(sys.argv)



# -*- coding: utf-8 -*-

__author__ = 'churas'

import logging
import os
import time
from d3r.celpp.task import D3RTask
from d3r.celpp import util
from d3r.celpp.makeblastdb import MakeBlastDBTask

logger = logging.getLogger(__name__)


class ImportRetryCountExceededError(Exception):
    """Error raised when import retry count has been exceeded
    """
    pass


class DataImportTask(D3RTask):
    """Represents DataImport Task
    This task downloads 3 files named new_release_structure_nonpolymer.tsv,
    new_release_structure_sequence.tsv, and
    new_release_crystallization_pH.tsv from the web
    """
    PARTICIPANT_LIST_CSV = "participant_list.csv"
    NONPOLYMER_TSV = "new_release_structure_nonpolymer.tsv"
    OLDSEQUENCE_TSV = "new_release_structure_sequence.tsv"
    SEQUENCE_TSV = "new_release_structure_sequence_canonical.tsv"
    CRYSTALPH_TSV = "new_release_crystallization_pH.tsv"
    COMPINCHI_ICH = "Components-inchi.ich"

    # standard to append to NON_POLYMER_TSV file
    NONPOLYMER_TSV_STANDARD = "1FCZ    156     InChI=1S/C24H26O3/c1-23(2)13-" \
                              "14-24(3,4)20-15-18(10-11-19(20)23)21(25)12-7-" \
                              "16-5-8-17(9-6-16)22(26)27/h5-12,15H,13-14H2,1" \
                              "-4H3,(H,26,27)/b12-7+\n"

    # standard to append to CRYSTALPH_TSV
    CRYSTALPH_TSV_STANDARD = "1FCZ	7\n"

    # standard to append to SEQUENCE_TSV
    SEQUENCE_TSV_STANDARD = "1FCZ    1       SPQLEELITKVSKAHQETFPSLCQLGKYTTN" \
                            "SSADHRVQLDLGLWDKFSELATKCIIKIVEFAKRLPGFTGLSIADQI" \
                            "TLLKAACLDILMLRICTRYTPEQDTMTFSDGLTLNRTQMHNAGFGPL" \
                            "TDLVFAFAGQLLPLEMDDTETGLLSAICLICGDRMDLEEPEKVDKLQ" \
                            "EPLLEALRLYARRRRPSQPYMFPRMLMKITDLRGISTKGAERAITLK" \
                            "MEIPGPMPPLIREMLE\n"

    TASK_NAME = 'dataimport'

    def __init__(self, path, args):
        super(DataImportTask, self).__init__(path, args)
        self.set_name(DataImportTask.TASK_NAME)
        makeblast = MakeBlastDBTask('', args)
        self.set_stage(makeblast.get_stage() + 1)
        self.set_status(D3RTask.UNKNOWN_STATUS)
        self._maxretries = 3
        self._retrysleep = 1

    def get_uploadable_files(self):
        """Returns list of files that can be uploaded to remote server

           List will contain these files in addition to stderr/stdout files
           if they are found on the filesystem

           new_release_structure_nonpolymer.tsv
           new_release_structure_sequence.tsv
           new_release_crystallization_pH.tsv
           Components-inchi.ich

           :returns: list of files that can be uploaded.
        """
        # get the stderr/stdout files
        file_list = super(DataImportTask, self).get_uploadable_files()

        compinchi = self.get_components_inchi_file()

        if os.path.isfile(compinchi):
            file_list.append(compinchi)

        crystal = self.get_crystalph_tsv()

        if os.path.isfile(crystal):
            file_list.append(crystal)

        nonpoly = self.get_nonpolymer_tsv()

        if os.path.isfile(nonpoly):
            file_list.append(nonpoly)

        seq = self.get_sequence_tsv()

        if os.path.isfile(seq):
            file_list.append(seq)

        return file_list

    def get_nonpolymer_tsv(self):
        """Returns path to new_release_structure_nonpolymer.tsv file
        :return: full path to DataImportTask.NONPOLYMER_TSV file
        """
        return os.path.join(self.get_dir(),
                            DataImportTask.NONPOLYMER_TSV)

    def get_oldsequence_tsv(self):
        """Returns path to new_release_structure_sequence.tsv file
        :return: full path to DataImportTask.SEQUENCE_TSV file
        """
        return os.path.join(self.get_dir(),
                            DataImportTask.OLDSEQUENCE_TSV)

    def get_sequence_tsv(self):
        """Returns path to new_release_structure_sequence_canonical.tsv file
        :return: full path to DataImportTask.SEQUENCE_TSV file
        """
        return os.path.join(self.get_dir(),
                            DataImportTask.SEQUENCE_TSV)

    def get_participant_list_csv(self):
        """Returns path to participant_list.csv file
        :returns: full path to DataImportTask.PARTICIPANT_LIST_CSV file
        """
        return os.path.join(self.get_dir(),
                            DataImportTask.PARTICIPANT_LIST_CSV)

    def get_crystalph_tsv(self):
        """Returns path to new_release_crystallization_pH.tsv file
        :return: full path to DataImportTask.CRYSTALPH_TSV file
        """
        return os.path.join(self.get_dir(),
                            DataImportTask.CRYSTALPH_TSV)

    def get_components_inchi_file(self):
        return os.path.join(self.get_dir(),
                            DataImportTask.COMPINCHI_ICH)

    def get_set_of_pdbid_from_crystalph_tsv(self):
        """Gets set of PDBID by parsing `DataImportTask.CRYSTALPH_TSV`

           Parses `DataImportTask.CRYSTALPH_TSV` to build a set of PDBIDs
           from the first 4 characters on each line minus the first line.

           Format space between columns appears to be tab:

            PDB_ID	_exptl_crystal_grow.pH
            <PDBID>	<PH>


           Example file:
            PDB_ID	_exptl_crystal_grow.pH
            4RFR	7.5
            4X09	6.5
            4XET	6.2
            4XF1	6.2
            4XF3	6.2

           :returns: set of PDBID in uppercase
        """
        pdbid_set = set()
        start_time = int(time.time())
        logger.debug('Examing ' + self.get_crystalph_tsv() +
                     ' to get set of PDBIDs')

        if not os.path.isfile(self.get_crystalph_tsv()):
            logger.warning('')
            return set()
        try:

            f = open(self.get_crystalph_tsv(), 'r')

            headerline = f.readline()
            if not headerline.startswith('PDB_ID'):
                logger.warning('First line in ' + self.get_crystalph_tsv() +
                               'did NOT start with PDBID_ID ('
                               + headerline + ')')
            for line in f:
                    pdbid_split = line.split()
                    if len(pdbid_split) != 2:
                        logger.error('Skipping entry... whitespace'
                                     ' split resulted in:' +
                                     str(len(pdbid_split)) +
                                     ' instead of expected 2: ' +
                                     line)
                        continue

                    pdbid_set.add(pdbid_split[0].upper())
            f.close()
            logger.debug('Found ' + str(len(pdbid_set)) +
                         ' PDBIDs from crystalph.  Parsing took ' +
                         str(int(time.time()) - start_time) +
                         ' seconds.')
            return pdbid_set
        except:
            logger.exception('Caught exception trying to get set of PDBIDs')
        return set()

    def get_set_of_pdbid_in_crystalph_tsv_and_pdb_seqres(self):
        """Gets set of PDBIDs that are in both tsv and sequence file

           Examines `DataImportTask.CRYSTALPH_TSV` and
           `MakeBlastDBTask.PDB_SEQRES_TXT` and returns a set of PDBIDs
           that are in both files
           :returns: set of PDBIDs uppercase that are in both files above
        """
        make_blastdb = MakeBlastDBTask(self._path, self._args)

        if not os.path.isfile(make_blastdb.get_pdb_seqres_txt()):
            logger.warning('No ' + make_blastdb.get_pdb_seqres_txt() +
                           ' file found')
            return set()

        c_pdbid_set = self.get_set_of_pdbid_from_crystalph_tsv()

        if len(c_pdbid_set) == 0:
            logger.warning('No PDBIds found in ' + self.get_crystalph_tsv())
            return set()

        seq_pdbid_set = make_blastdb.get_set_of_pbdid_from_pdb_seqres_txt()

        if len(seq_pdbid_set) == 0:
            logger.warning('No PDBIds found in ' +
                           make_blastdb.get_pdb_seqres_txt())
            return set()

        common_pdbid = set()

        # iterate through tsv pdb ids and return any found in
        # sequence pdb id set
        for id in c_pdbid_set:
            if id in seq_pdbid_set:
                common_pdbid.add(id)

        logger.debug('Found ' + str(len(common_pdbid)) + ' PDBIDs in ' +
                     self.get_crystalph_tsv() + ' and ' +
                     make_blastdb.get_pdb_seqres_txt())

        return common_pdbid

    def append_standard_to_files(self):
        """Appends standard 1FCZ to tsv files as mapped below

           NONPOLYMER_TSV_STANDARD is appended to `get_nonpolymer_tsv()`
           CRYSTALPH_TSV_STANDARD is appended to `get_crystalph_tsv()`
           SEQUENCE_TSV_STANDARD is appended to `get_sequence_tsv()`
           SEQUENCE_TSV_STANDARD is appended to `get_oldsequence_tsv()`
        """
        logger.debug('Appending 1FCZ standard to tsv files')
        try:
            util.append_string_to_file(self.get_nonpolymer_tsv(),
                                       DataImportTask.NONPOLYMER_TSV_STANDARD)
            util.append_string_to_file(self.get_sequence_tsv(),
                                       DataImportTask.SEQUENCE_TSV_STANDARD)
            util.append_string_to_file(self.get_oldsequence_tsv(),
                                       DataImportTask.SEQUENCE_TSV_STANDARD)
            util.append_string_to_file(self.get_crystalph_tsv(),
                                       DataImportTask.CRYSTALPH_TSV_STANDARD)

            self.append_to_email_log('\nAppending 1FCZ standard to tsv'
                                     ' files\n')
            nonpoly_count = util.get_file_line_count(self.get_nonpolymer_tsv())
            seq_count = util.get_file_line_count(self.get_sequence_tsv())
            crystal_count = util.get_file_line_count(self.get_crystalph_tsv())
            inchi_count = util.get_file_line_count(self.
                                                   get_components_inchi_file())

            self.append_to_email_log('Line counts:\n')
            self.append_to_email_log(str(inchi_count) + ' ' +
                                     DataImportTask.COMPINCHI_ICH + '\n')

            self.append_to_email_log(str(nonpoly_count) + ' ' +
                                     DataImportTask.NONPOLYMER_TSV + '\n')

            self.append_to_email_log(str(seq_count) + ' ' +
                                     DataImportTask.SEQUENCE_TSV + '\n')
            self.append_to_email_log(str(seq_count) + ' ' +
                                     DataImportTask.OLDSEQUENCE_TSV + '\n')
            self.append_to_email_log(str(crystal_count) + ' ' +
                                     DataImportTask.CRYSTALPH_TSV + '\n')

        except IOError:
            logger.exception('Error appending standard to file')

    def _wait_for_url_to_be_updated(self, url):
        """Waits for `url` to have been updated since start of celpp week
        """
        try:
            if self.get_args().skipimportwait is True:
                logger.debug('Skiping wait for tsv files to be updated')
                return
        except Exception:
            logger.exception('Skiping wait for tsv files to be updated')
            return

        try:
            importsleep = int(self.get_args().importsleep)
        except Exception:
            logger.exception('importsleep was not set using 1 second')
            importsleep = 1

        val = util.has_url_been_updated_since_start_of_celpp_week(url)
        counter = 0
        while val is False:
            if counter > self.get_args().importretry:
                raise ImportRetryCountExceededError(
                    url + ' has not been updated after ' +
                    str(counter*importsleep) +
                    ' seconds')
            logger.debug('Try #' + str(counter) + ' ' + url +
                         ' has not been updated. Sleeping ' +
                         str(importsleep) + ' seconds')
            time.sleep(importsleep)

            val = util.has_url_been_updated_since_start_of_celpp_week(url)
            counter += 1

    def can_run(self):
        """Determines if task can actually run

           The method verifies a `DataImportTask` does
           not already exist.  If above is not true then self.set_error()
           is set with information about the issue
           :return: True if can run otherwise False
        """
        self._can_run = False
        self._error = None

        # check this task is not complete and does not exist

        self.update_status_from_filesystem()
        if self.get_status() == D3RTask.COMPLETE_STATUS:
            logger.debug("No work needed for " + self.get_name() +
                         " task")
            return False

        make_blastdb = MakeBlastDBTask(self._path, self._args)
        make_blastdb.update_status_from_filesystem()
        if make_blastdb.get_status() != D3RTask.COMPLETE_STATUS:
            logger.info('Cannot run ' + self.get_name() + ' task ' +
                        'because ' + make_blastdb.get_name() + ' task' +
                        'has a status of ' + make_blastdb.get_status())
            self.set_error(make_blastdb.get_name() + ' task has ' +
                           make_blastdb.get_status() + ' status')
            return False

        if self.get_status() != D3RTask.NOTFOUND_STATUS:
            logger.warning(self.get_name() + " task was already " +
                           "attempted, but there was a problem")
            self.set_error(self.get_dir_name() + ' already exists and ' +
                           'status is ' + self.get_status())
            return False
        self._can_run = True
        return True

    def _download_files(self, url):

        try:
            nonpolyurl = url + '/' + DataImportTask.NONPOLYMER_TSV

            self._wait_for_url_to_be_updated(nonpolyurl)

            download_path = self.get_nonpolymer_tsv()
            util.download_url_to_file(nonpolyurl,
                                      download_path,
                                      self._maxretries,
                                      self._retrysleep)

            sequence = url + '/' + DataImportTask.SEQUENCE_TSV
            self._wait_for_url_to_be_updated(sequence)

            download_path = self.get_sequence_tsv()
            util.download_url_to_file(sequence,
                                      download_path,
                                      self._maxretries,
                                      self._retrysleep)

            oldsequence = url + '/' + DataImportTask.OLDSEQUENCE_TSV
            self._wait_for_url_to_be_updated(oldsequence)

            download_path = self.get_oldsequence_tsv()
            util.download_url_to_file(oldsequence,
                                      download_path,
                                      self._maxretries,
                                      self._retrysleep)

            crystal = url + '/' + DataImportTask.CRYSTALPH_TSV

            self._wait_for_url_to_be_updated(crystal)
            download_path = self.get_crystalph_tsv()
            util.download_url_to_file(crystal,
                                      download_path,
                                      self._maxretries,
                                      self._retrysleep)

            url = self._args.compinchi
            download_path = self.get_components_inchi_file()
            util.download_url_to_file(url +
                                      '/' + DataImportTask.COMPINCHI_ICH,
                                      download_path,
                                      self._maxretries,
                                      self._retrysleep)
            return True
        except Exception:
            logger.exception('Caught Exception trying to download file(s)')

            self.set_error('Unable to download file from ' +
                           url + ' to ' + download_path)
            # assess the result
            self.end()
        return False

    def _download_participant_list_csv(self):
        """Downloads from ftp the DataImportTask.PARTICIPANT_LIST_CSV file
           If there is an error. A message is appended to email log, but its
           not considered a failure
        """
        localfile = self.get_participant_list_csv()

        try:
            ft = self.get_file_transfer()
            ft.connect()
            remotefile = os.path.join(ft.get_remote_dir(),
                                      DataImportTask.PARTICIPANT_LIST_CSV)
            logger.debug('Downloading ' + remotefile + ' to ' +
                         localfile)
            ft.download_file(remotefile,
                             localfile)
        except Exception as e:
            logger.warning('Caught exception, but not considering this fatal')
            self.append_to_email_log('\nWARNING: Unable to download ' +
                                     DataImportTask.PARTICIPANT_LIST_CSV +
                                     ' which means external users will NOT '
                                     'get evaluation email : ' + str(e) + '\n')
            return
        finally:
            try:
                ft.disconnect()
            except Exception as e:
                logger.warning('Caught exception trying to disconnect')
        if not os.path.isfile(localfile):
            self.append_to_email_log('\nWARNING: ' +
                                     DataImportTask.PARTICIPANT_LIST_CSV +
                                     ' not downloaded which means external '
                                     'users will NOT get evaluation email\n')

    def run(self):
        """Downloads Components-inchi.ich

           Downloads 3 files from url specified in `self._args.pdbfileurl`
           namely the ones mentioned in the constructor to get_dir() directory.
           Each download will be retried up to self._maxretries time
           which is set in constructor.  After which set_error()
           will be set with message if file is still unable to be downloaded.
           """
        super(DataImportTask, self).run()

        try:
            logger.debug('pdbfileurl set to ' + self._args.pdbfileurl)
        except AttributeError:
            self.set_error('cannot download files cause pdbfileurl not set')
            self.end()
            return

        try:
            logger.debug('compinchi set to ' + self._args.compinchi)
        except AttributeError:
            self.set_error('cannot download files cause compinchi not set')
            self.end()
            return

        if self._can_run is False:
            logger.debug(
                self.get_dir_name() + ' cannot run cause _can_run flag '
                                      'is False')
            return

        url = self._args.pdbfileurl

        # if False is returned then the download failed
        # just return since the method has already
        # called end() and set_error()
        if self._download_files(url) is False:
            return

        # download the participant_list.csv file
        # if its missing no biggy
        self._download_participant_list_csv()

        # Compare TSV files with pdb_seqres.txt file to see if there
        # are any duplicates issue #15
        try:
            pdbs = self.get_set_of_pdbid_in_crystalph_tsv_and_pdb_seqres()
            if len(pdbs) == 0:
                self.append_to_email_log('\nFound no entries in ' +
                                         DataImportTask.CRYSTALPH_TSV +
                                         'and ' +
                                         MakeBlastDBTask.PDB_SEQRES_TXT +
                                         ' files\n')
            else:
                self.append_to_email_log('\nWARNING: Found ' +
                                         str(len(pdbs)) +
                                         ' PDBIDs in both ' +
                                         DataImportTask.CRYSTALPH_TSV +
                                         ' and ' +
                                         MakeBlastDBTask.PDB_SEQRES_TXT +
                                         ' files\n')
        except Exception:
            logger.exception('Caught exception comparing tsv with '
                             'sequence')
            self.append_to_email_log('\nError unable to examine ' +
                                     DataImportTask.CRYSTALPH_TSV +
                                     ' and/or ' +
                                     MakeBlastDBTask.PDB_SEQRES_TXT +
                                     ' files\n')

        # Append internal standard
        self.append_standard_to_files()
        self.end()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_task
----------------------------------

Tests for `task` module.
"""
import unittest
import tempfile
import shutil
import platform
import os
import pwd
import gzip
from email.mime.multipart import MIMEMultipart

from mock import Mock

from d3r.celpp.task import D3RParameters
from d3r.celpp.task import UnsetPathError
from d3r.celpp.task import UnsetStageError
from d3r.celpp.task import UnsetNameError
from d3r.celpp.task import UnsetCommandError
from d3r.celpp.task import UnsetFileNameError
from d3r.celpp.task import D3RTask
from d3r.celpp.task import SmtpEmailer
from d3r.celpp.task import EmailSendError
from d3r.celpp.task import Attachment
from d3r.celpp.filetransfer import FtpFileTransfer


class MockException(Exception):
    pass


class MockFileTransfer(FtpFileTransfer):
    """Fake FileUploader
    """
    def __init__(self):
        self._upload_files_except_object = None
        self._upload_summary_except_object = None
        self._upload_summary = None
        self._upload_files = None
        self._connect = True

    def set_upload_files(self, val):
        self._upload_files = val

    def set_upload_files_exception(self, except_object):
        self._upload_files_except_object = except_object

    def upload_files(self, list_of_files):
        """

        :param list_of_files:
        :return:
        """
        if self._upload_files_except_object is not None:
            raise self._upload_files_except_object
        return self._upload_files

    def set_upload_summary(self, val):
        self._upload_summary = val

    def set_upload_summary_exception(self, except_object):
        self._upload_summary_except_object = except_object

    def get_upload_summary(self):
        """

        :return: String set in set_upload_summary()
        """
        if self._upload_summary_except_object is not None:
            raise self._upload_summary_except_object

        return self._upload_summary

    def set_connect(self, val):
        self._connect = val

    def connect(self):
        return self._connect

    def disconnect(self):
        pass


class TestD3rTask(unittest.TestCase):

    def setUp(self):
        pass

    def test_constructor(self):
        params = D3RParameters()
        task = D3RTask('/path', params)
        self.assertEqual(task.get_name(), None)
        self.assertEqual(task.get_path(), '/path')
        self.assertEqual(task.get_stage(), None)
        self.assertEqual(task.get_status(), D3RTask.UNKNOWN_STATUS)
        self.assertEqual(task.get_error(), None)
        task.set_name('foo')
        task.set_path('blah')
        task.set_stage(4)
        task.set_status(D3RTask.START_STATUS)
        task.set_error('error')
        task.set_file_transfer('yah')

        self.assertEqual(task.get_name(), 'foo')
        self.assertEqual(task.get_path(), 'blah')
        self.assertEqual(task.get_stage(), 4)
        self.assertEqual(task.get_status(), D3RTask.START_STATUS)
        self.assertEqual(task.get_error(), 'error')
        self.assertEqual(task.get_file_transfer(), 'yah')

        params.ftpconfig = '/somefile'
        task = D3RTask('/path', params)
        self.assertEqual(task.get_file_transfer(), None)

    def test_get_dir_name(self):
        params = D3RParameters()
        task = D3RTask(None, params)
        try:
            task.get_dir_name()
            self.fail('Expected UnsetStageError')
        except UnsetStageError:
            pass

        task.set_stage(1)
        try:
            task.get_dir_name()
            self.fail('Expected UnsetNameError')
        except UnsetNameError:
            pass

        task.set_name('foo')
        self.assertEqual(task.get_dir_name(), 'stage.1.foo')

    def test_get_dir(self):
        params = D3RParameters()
        task = D3RTask(None, params)
        try:
            task.get_dir()
            self.fail('Expected UnsetPathError')
        except UnsetPathError:
            pass

        task.set_path('/blah')

        try:
            task.get_dir()
            self.fail('Expected UnsetStageError')
        except UnsetStageError:
            pass

        task.set_stage(1)
        try:
            task.get_dir()
            self.fail('Expected UnsetNameError')
        except UnsetNameError:
            pass

        task.set_name('foo')

        self.assertEqual(task.get_dir(), '/blah/stage.1.foo')

    def test_get_set_email_log(self):
        params = D3RParameters()
        task = D3RTask(None, params)
        self.assertEqual(task.get_email_log(), None)
        task.set_email_log('hi')
        self.assertEqual(task.get_email_log(), 'hi')

    def test_get_program_name(self):
        params = D3RParameters()
        task = D3RTask(None, params)
        self.assertTrue(task._get_program_name().find('task.py') > 0,
                        task._get_program_name())
        params.program = 'proggy'
        params.version = 'versy'
        task = D3RTask(None, params)
        self.assertEqual(task._get_program_name(), 'proggy versy')

    def test_get_program_version(self):
        params = D3RParameters()
        task = D3RTask(None, params)
        self.assertEqual(task._get_program_version(), '')
        params.version = '0.1.1'
        task = D3RTask(None, params)
        self.assertEqual(task._get_program_version(), '0.1.1')

    def test_get_uploadable_files(self):
        task = D3RTask(None, D3RParameters())
        self.assertEqual(task.get_uploadable_files(), [])
        temp_dir = tempfile.mkdtemp()
        try:
            task = D3RTask(temp_dir, D3RParameters())
            task.set_stage(1)
            task.set_name('foo')
            task.create_dir()
            self.assertEqual(task.get_uploadable_files(), [])

            # add error file
            err_file = os.path.join(task.get_dir(), D3RTask.ERROR_FILE)
            open(err_file, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 1)
            self.assertEqual(flist[0], err_file)

            # add stderr file
            foostderr = os.path.join(task.get_dir(), 'foo.stderr')
            open(foostderr, 'a').close()
            flist = task.get_uploadable_files()
            flist.index(err_file)
            flist.index(foostderr)
            self.assertEqual(len(flist), 2)

            # add stdout file
            foostdout = os.path.join(task.get_dir(), 'foo.stdout')
            open(foostdout, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 3)
            flist.index(err_file)
            flist.index(foostderr)
            flist.index(foostdout)
        finally:
            shutil.rmtree(temp_dir)

    def test_can_run(self):
        task = D3RTask(None, D3RParameters())
        self.assertEqual(task.can_run(), False)

    def test_upload_task(self):
        task = D3RTask(None, D3RParameters())
        self.assertEqual(task._file_uploader, None)
        # try calling upload_task where _file_uploader is None
        task._upload_task()

        # try calling upload_task where _file_uploader is NOT None
        params = D3RParameters()
        params.ftpconfig = 'foo'
        task = D3RTask(None, params)
        self.assertEqual(task._file_uploader, None)
        # try calling upload_task where _file_uploader is None
        task._upload_task()

        # try calling upload_task where _file_uploader.upload_files
        # throws exception
        uploader = MockFileTransfer()
        uploader.set_upload_files_exception(MockException('hi'))
        task = D3RTask(None, D3RParameters())
        task.set_file_transfer(uploader)
        task._upload_task()

        # try calling where upload_summary throws exception
        uploader.set_upload_files_exception(None)
        uploader.set_upload_summary_exception(MockException('boo'))
        task._upload_task()

        # try calling where upload_summary returns None
        uploader.set_upload_summary_exception(None)
        uploader.set_upload_summary(None)
        task._upload_task()

        uploader.set_upload_summary('hello')
        task._upload_task()
        self.assertEqual(task._email_log, '\nhello\n')

    def test_run(self):
        params = D3RParameters()
        task = D3RTask(None, params)
        self.assertEqual(task._can_run, None)
        task.run()
        self.assertEqual(task._can_run, False)
        task._can_run = True

        temp_dir = tempfile.mkdtemp()
        task.set_name('foo')
        task.set_stage(1)
        task.set_path(temp_dir)
        try:
            task.run()
        finally:
            shutil.rmtree(temp_dir)

    def test_write_to_file(self):
        tempDir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = D3RTask(None, params)
            try:
                task.write_to_file('hello', None)
                self.fail('Expected UnsetFileNameError')
            except UnsetFileNameError:
                pass
            try:
                task.write_to_file('hello', 'foo')
                self.fail('Expected UnsetPathError')
            except UnsetPathError:
                pass
            task.set_name('foo')
            task.set_stage(1)
            task.set_path(tempDir)

            try:
                task.write_to_file('hello', 'foo')
                self.fail('Expected IOError')
            except IOError:
                pass
            task.create_dir()
            task.write_to_file('hello', 'foo')
            self.assertEqual(os.path.isfile(os.path.join(task.get_dir(),
                                            'foo')), True)
        finally:
            shutil.rmtree(tempDir)

    def test_create_dir(self):
        tempDir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = D3RTask(None, params)
            try:
                task.create_dir()
                self.fail('Expected UnsetPathError')
            except UnsetPathError:
                pass
            task.set_name('foo')
            task.set_stage(1)
            task.set_path(tempDir)
            self.assertEqual(task.create_dir(),
                             os.path.join(tempDir, 'stage.1.foo'))
        finally:
            shutil.rmtree(tempDir)

    def test_update_status_from_filesystem(self):
        params = D3RParameters()
        task = D3RTask(None, params)

        try:
            task.update_status_from_filesystem()
            self.fail("Expected UnsetPathError")
        except UnsetPathError:
            pass

        tempDir = tempfile.mkdtemp()
        try:
            task.set_path(tempDir)

            # Test unset stage
            try:
                task.update_status_from_filesystem()
                self.fail("Expected UnsetStageError")
            except UnsetStageError:
                pass
        finally:
            shutil.rmtree(tempDir)

        task.set_stage(1)
        task.set_name('foo')

        try_update_status_from_filesystem(self, task)

    def test_start_no_version(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = D3RTask(temp_dir, params)
            task.set_stage(1)
            task.set_name('foo')
            task.start()
            self.assertEqual(os.path.isfile(os.path.join(task.get_dir(),
                                            D3RTask.START_FILE)), True)
            self.assertEqual(task.get_error(), None)
            self.assertEqual(task.get_status(), D3RTask.START_STATUS)
            task.start()
            self.assertNotEqual(task.get_error(), None)
            self.assertEqual(task.get_status(), D3RTask.ERROR_STATUS)
            self.assertEqual(os.path.isfile(os.path.join(task.get_dir(),
                                                         D3RTask
                                                         .ERROR_FILE)), True)

        finally:
            shutil.rmtree(temp_dir)

    def test_start_with_version(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.version = '1.6.0'
            task = D3RTask(temp_dir, params)
            task.set_stage(1)
            task.set_name('foo')
            task.start()
            sfile = os.path.join(task.get_dir(),
                                 D3RTask.START_FILE)

            self.assertEqual(os.path.isfile(sfile), True)
            f = open(sfile, 'r')
            self.assertEqual(f.read(), '1.6.0')
            self.assertEqual(task.get_error(), None)
            self.assertEqual(task.get_status(), D3RTask.START_STATUS)
            task.start()
            self.assertNotEqual(task.get_error(), None)
            self.assertEqual(task.get_status(), D3RTask.ERROR_STATUS)
            self.assertEqual(os.path.isfile(os.path.join(task.get_dir(),
                                                         D3RTask
                                                         .ERROR_FILE)), True)

        finally:
            shutil.rmtree(temp_dir)

    def test_end(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = D3RTask(temp_dir, params)
            task.set_stage(1)
            task.set_name('foo')
            task.start()
            task.end()
            self.assertEqual(os.path.isfile(os.path.join(task.get_dir(),
                                                         D3RTask.
                                                         COMPLETE_FILE)), True)
            self.assertEqual(task.get_error(), None)
            self.assertEqual(task.get_status(), D3RTask.COMPLETE_STATUS)
            task.set_error('some error')
            task.end()
            self.assertEqual(task.get_error(), 'some error')
            self.assertEqual(task.get_status(), D3RTask.ERROR_STATUS)

            err_file = os.path.join(task.get_dir(), D3RTask.ERROR_FILE)

            self.assertEqual(os.path.isfile(err_file), True)
            f = open(err_file, 'r')
            err_msg = f.readline()
            self.assertEqual(err_msg, 'some error\n')
            task.set_status(D3RTask.ERROR_STATUS)
            os.remove(err_file)
            task.set_error(None)
            task.end()
            self.assertEqual(task.get_status(), D3RTask.ERROR_STATUS)
            self.assertEqual(os.path.isfile(err_file), True)
            self.assertEqual(os.path.getsize(err_file), 0)

        finally:
            shutil.rmtree(temp_dir)

    def test_send_email_where_email_is_none(self):
        params = D3RParameters()
        params.email = None
        task = D3RTask('/foo', params)
        task.set_stage(1)
        task.set_name('foo')
        task._send_end_email()

    def test_send_email_throws_exception(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.smtp = 'doesnotexistxxx.x.x'
            params.smtpport = 12345
            task = D3RTask(temp_dir, params)
            task.set_stage(1)
            task.set_name('foo')
            try:
                task._send_email(None, 'bob@bob.com')
                self.fail('Expected exception')
            except Exception:
                pass
        finally:
            shutil.rmtree(temp_dir)

    def test_send_end_email_throws_exception(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.smtp = 'doesnotexistxxx.x.x'
            params.smtpport = 12345
            params.email = 'foo@foo.com'
            task = D3RTask(temp_dir, params)
            task.set_stage(1)
            task.set_name('foo')
            task._send_end_email()

        finally:
            shutil.rmtree(temp_dir)

    def test_get_mail_truncated_string_val_is_none(self):
        params = D3RParameters()
        task = D3RTask('/foo', params)
        self.assertEqual(task._get_email_truncated_string(None,
                                                          100), None)

    def test_get_mail_truncated_string_maxchars_none_or_negative(self):
        params = D3RParameters()
        task = D3RTask('/foo', params)
        self.assertEqual(task._get_email_truncated_string('foo',
                                                          None), 'foo')

        self.assertEqual(task._get_email_truncated_string('foo',
                                                          -1), 'foo')

    def test_get_mail_truncated_string_truncated(self):
        params = D3RParameters()
        task = D3RTask('/foo', params)
        val = '123456789 abcdefghijklmnop'
        res = task._get_email_truncated_string(val, 15)
        self.assertEqual(res, D3RTask.TEXT_TRUNCATED_STR +
                         'bcdefghijklmnop')

    def test_get_smtp_server(self):
        params = D3RParameters()
        params.smtp = ''
        params.smtpport = '25'
        task = D3RTask(None, params)
        self.assertNotEqual(task._get_smtp_server(), None)

    def test_build_from_address(self):
        params = D3RParameters()
        task = D3RTask(None, params)
        exp_from_addr = pwd.getpwuid(os.getuid())[0] + '@' + platform.node()
        self.assertEqual(task._build_from_address(), exp_from_addr)

    def test_run_external_command_all_params_None(self):
        params = D3RParameters()
        task = D3RTask(None, params)
        try:
            task.run_external_command(None, None, None)
            self.fail('expected UnsetNameError')
        except UnsetNameError:
            pass

    def test_run_external_command_name_None(self):
        params = D3RParameters()
        task = D3RTask(None, params)
        try:
            task.run_external_command(None, 'true', None)
            self.fail('expected UnsetNameError')
        except UnsetNameError:
            pass

    def test_run_external_command_name_cmd_to_run_None(self):
        params = D3RParameters()
        task = D3RTask(None, params)
        try:
            task.run_external_command('hello', None, None)
            self.fail('expected UnsetCommandError')
        except UnsetCommandError:
            pass

    def test_run_external_command_name_failure_is_fatal_is_None(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = D3RTask(None, params)
            task.set_name('foo')
            task.set_stage(1)
            task.set_path(temp_dir)
            task.create_dir()
            task.set_path(temp_dir)
            self.assertEquals(1, task.run_external_command('hi',
                                                           'false',
                                                           None))
            self.assertEquals(task.get_error(), 'Non zero exit code:' +
                              ' 1 received. Standard out:  Standard error: ')
            self.assertEquals(task.get_status(), D3RTask.ERROR_STATUS)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_external_command_cmd_raises_exception(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = D3RTask(None, params)
            task.set_name('foo')
            task.set_stage(1)
            task.set_path(temp_dir)
            task.create_dir()
            task.set_path(temp_dir)
            self.assertEquals(1, task.run_external_command('hi',
                                                           'asdfasdf',
                                                           None))
            self.assertEquals(task.get_error(), 'Caught Exception trying ' +
                              'to run asdfasdf : [Errno 2] No such file ' +
                              'or directory')

            self.assertEquals(os.path.exists(os.path.join(task.get_dir(),
                                                          'hi.stdout')),
                              False)
            self.assertEquals(os.path.exists(os.path.join(task.get_dir(),
                                                          'hi.stderr')),
                              False)

        finally:
            shutil.rmtree(temp_dir)

    def test_run_external_command_cmd_fails_and_is_fatal(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = D3RTask(None, params)
            task.set_name('foo')
            task.set_stage(1)
            task.set_path(temp_dir)
            task.create_dir()
            task.set_path(temp_dir)
            self.assertEquals(1, task.run_external_command('hi',
                                                           'false',
                                                           True))
            self.assertEquals(task.get_error(), 'Non zero exit code:' +
                              ' 1 received. Standard out:  Standard error: ')
            self.assertEquals(task.get_status(), D3RTask.ERROR_STATUS)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_external_command_cmd_fails_and_isnot_fatal(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = D3RTask(None, params)
            task.set_name('foo')
            task.set_stage(1)
            task.set_path(temp_dir)
            task.create_dir()
            task.set_path(temp_dir)
            self.assertEquals(1, task.run_external_command('hi',
                                                           'false',
                                                           False))
            self.assertEquals(task.get_error(), None)
            self.assertEquals(task.get_status(), D3RTask.UNKNOWN_STATUS)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_external_command_cmd_succeeds(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = D3RTask(None, params)
            task.set_name('foo')
            task.set_stage(1)
            task.set_path(temp_dir)
            task.create_dir()
            task.set_path(temp_dir)
            self.assertEquals(0, task.run_external_command('hi',
                                                           'echo hi',
                                                           False))
            self.assertEquals(task.get_error(), None)
            self.assertEquals(task.get_status(), D3RTask.UNKNOWN_STATUS)
            self.assertEquals(os.path.exists(os.path.join(task.get_dir(),
                                                          'hi.stdout')),
                              True)
            self.assertEquals(os.path.exists(os.path.join(task.get_dir(),
                                                          'hi.stderr')),
                              True)

        finally:
            shutil.rmtree(temp_dir)

    def test_run_external_command_withtimeout_cmd_succeeds(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = D3RTask(None, params)
            task.set_name('foo')
            task.set_stage(1)
            task.set_path(temp_dir)
            task.create_dir()
            task.set_path(temp_dir)
            self.assertEquals(0, task.run_external_command('hi',
                                                           'echo hi',
                                                           False,
                                                           timeout=30))
            self.assertEquals(task.get_error(), None)
            self.assertEquals(task.get_status(), D3RTask.UNKNOWN_STATUS)
            self.assertEquals(os.path.exists(os.path.join(task.get_dir(),
                                                          'hi.stdout')),
                              True)
            self.assertEquals(os.path.exists(os.path.join(task.get_dir(),
                                                          'hi.stderr')),
                              True)

        finally:
            shutil.rmtree(temp_dir)

    def test_smtp_emailer_generate_from_address_using_login_and_host(self):
        emailer = SmtpEmailer()
        val = emailer.generate_from_address_using_login_and_host()
        self.assertEqual(val, pwd.getpwuid(os.getuid())[0] + '@' +
                         platform.node())

    def test_sending_real_email_but_to_invalid_port_and_host(self):
        emailer = SmtpEmailer(port=1231231231232)
        try:
            emailer.send_email('bob@bob.com', ['joe@joe.com'], 'subby2',
                               'hi\n')
            self.fail('Expected EmailSendError')
        except EmailSendError:
            pass

    def test_get_server_with_altserver_set(self):
        emailer = SmtpEmailer()
        fake = D3RParameters()
        fake.hi = 'fake'
        emailer.set_alternate_smtp_server(fake)
        self.assertEqual(emailer._get_server().hi, fake.hi)

    def test_send_valid_email_to_fake_server(self):
        emailer = SmtpEmailer()
        mockserver = D3RParameters()
        mockserver.sendmail = Mock()
        mockserver.quit = Mock()

        emailer.set_alternate_smtp_server(mockserver)
        emailer.send_email('bob@bob.com', ['joe@joe.com'], 'subby2',
                           'hi\n', reply_to='rep@rep.com')
        mockserver.quit.assert_any_call()
        self.assertEqual(mockserver.sendmail.call_count, 1)

    def test_send_invalid_email_to_fake_server_that_throws_exception(self):
        emailer = SmtpEmailer()
        mockserver = D3RParameters()
        mockserver.sendmail = Mock(side_effect=IOError('some error'))
        mockserver.quit = Mock()

        emailer.set_alternate_smtp_server(mockserver)
        try:
            emailer.send_email('bob@bob.com', ['joe@joe.com'], 'subby2',
                               'hi\n', reply_to='rep@rep.com')
        except EmailSendError:
            pass

        mockserver.quit.assert_any_call()

    def test_attachment_class(self):
        a = Attachment('/foo', 'hi')
        self.assertEqual(a.get_desired_name(), 'hi')
        self.assertEqual(a.get_file_to_attach(), '/foo')

    def test_append_attachments(self):
        temp_dir = tempfile.mkdtemp()
        try:
            emailer = SmtpEmailer()
            # try no attachments
            msg_root = MIMEMultipart("alternative")
            res = emailer._append_attachments(msg_root, None)
            self.assertEqual(msg_root, res)

            # try 1 text attachment
            txtfile = os.path.join(temp_dir, 'my.txt')
            f = open(txtfile, 'w')
            f.write('hello')
            f.flush()
            f.close()
            msg_root = MIMEMultipart("alternative")
            a = Attachment(txtfile, 'renamed.txt')
            res = emailer._append_attachments(msg_root, [a])
            res.as_string().index('Content-Type: text/plain; '
                                  'charset="us-ascii"')
            res.as_string().index('Content-Disposition: attachment; '
                                  'filename="renamed.txt"')
            res.as_string().index('hello')

            # try 1 image attachment
            img = os.path.join(temp_dir, 'yo.ppm')
            f = open(img, 'w')
            f.write('P6\n1 1\n255\n')
            f.write('%c' % 255)
            f.flush()
            f.close()
            msg_root = MIMEMultipart("alternative")
            b = Attachment(img, None)
            res = emailer._append_attachments(msg_root, [b])
            res.as_string().index('Content-Type: image/x-portable-pixmap')
            res.as_string().index('Content-Disposition: attachment; '
                                  'filename="yo.ppm"')

            # try 1 gzip attachment
            mygz = os.path.join(temp_dir, 'foo.gz')
            f = gzip.open(mygz, 'wb')
            f.write('some compressed data')
            f.flush()
            f.close()
            c = Attachment(mygz, 'well.gz')
            res = emailer._append_attachments(msg_root, [c])
            res.as_string().index('Content-Type: application/octet-stream')
            res.as_string().index('Content-Disposition: attachment; '
                                  'filename="well.gz"')

            # try 3 attachments above together
            msg_root = MIMEMultipart("alternative")
            res = emailer._append_attachments(msg_root, [a, b, c])
            res.as_string().index('Content-Type: text/plain; '
                                  'charset="us-ascii"')
            res.as_string().index('Content-Disposition: attachment; '
                                  'filename="renamed.txt"')
            res.as_string().index('hello')

            res.as_string().index('Content-Type: image/x-portable-pixmap')
            res.as_string().index('Content-Disposition: attachment; '
                                  'filename="yo.ppm"')

            res.as_string().index('Content-Type: application/octet-stream')
            res.as_string().index('Content-Disposition: attachment; '
                                  'filename="well.gz"')
        finally:
            shutil.rmtree(temp_dir)

    def tearDown(self):
        pass


def try_update_status_from_filesystem(self, task):
    """Runs various tests on update_status_filesystem method
    This is a function cause this needs to be tested on
    all the subclasses as well
    """
    tempDir = tempfile.mkdtemp()
    try:
        task.set_path(tempDir)

        self.assertEqual(task.update_status_from_filesystem(),
                         D3RTask.NOTFOUND_STATUS)

        self.assertEqual(task.get_status(),
                         D3RTask.NOTFOUND_STATUS)

        # Test directory exists but no complete, start, or error files
        task.create_dir()
        self.assertEqual(task.update_status_from_filesystem(),
                         D3RTask.UNKNOWN_STATUS)

        self.assertEqual(task.get_status(),
                         D3RTask.UNKNOWN_STATUS)

        # Test start file exists
        startFile = os.path.join(tempDir, task.get_dir_name(),
                                 'start')
        open(startFile, 'a').close()

        self.assertEqual(task.update_status_from_filesystem(),
                         D3RTask.START_STATUS)

        self.assertEqual(task.get_status(),
                         D3RTask.START_STATUS)

        # Test error file exists
        errorFile = os.path.join(tempDir, task.get_dir_name(),
                                 'error')
        open(errorFile, 'a').close()

        self.assertEqual(task.update_status_from_filesystem(),
                         D3RTask.ERROR_STATUS)

        self.assertEqual(task.get_status(),
                         D3RTask.ERROR_STATUS)

        # Test complete file exists
        completeFile = os.path.join(tempDir, task.get_dir_name(),
                                    'complete')
        open(completeFile, 'a').close()

        self.assertEqual(task.update_status_from_filesystem(),
                         D3RTask.COMPLETE_STATUS)

        self.assertEqual(task.get_status(),
                         D3RTask.COMPLETE_STATUS)

    finally:
        shutil.rmtree(tempDir)


if __name__ == '__main__':
    unittest.main()
